@@ -1,6 +1,6 @@
 /* newlisp.h - header file for newLISP
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,26 +20,15 @@
 #ifndef NEWLISP_H
 #define NEWLISP_H
 
-/* include -DFFI in your makefile on the compile line 
-   and -lffi on the link line */
-#ifdef FFI
+/* Take out bigger, less used code portions.  By default these capabilities 
+   are enabled on all shipped binaries. Out-comment definitions to suppress.
+   Other capabilities like READLINE, SUPPORT_UTF8 can be en/disabled in
+   makefiles.
+*/
+#define XML_SUPPORT
+#define BIGINT
+#define KMEANS 
 
-#if defined(MAC_OSX)
-#include <ffi/ffi.h>
-#endif
-
-#if defined(WIN_32) || defined(CYGWN) 
-#include "win32-ffi.h" 
-#endif
-
-#ifdef LINUX /* makefiles specify include directory */
-#  include <ffi.h>
-#endif
-
-#define LIBFFI " libffi"
-#else /* not FFI */
-#define LIBFFI ""
-#endif /* FFI */
 
 /* config.h is only needed when doing auto configuration with ./configure-alt */
 #ifdef NEWCONFIG
@@ -48,17 +37,30 @@
 #define NEWLISPDIR "/usr/share/newlisp"
 #endif
 
+/* force ISO_C90 restrictions */
+#if defined(CYGWN) || defined(OS2) || defined(SOLARIS) || defined(AIX) || defined(SUNOS)
+#define ISO_C90
+#endif
+
 #ifdef LINUX
 #define OSTYPE "Linux"
 #endif
 
+#ifdef ANDROID
+#define NO_SEMAPHORE 
+#endif
+
 #ifdef _BSD
 #define OSTYPE "BSD"
 #endif
 
 #ifdef MAC_OSX
+#ifdef EMSCRIPTEN
+#define OSTYPE "JS"
+#else
 #define OSTYPE "OSX"
 #endif
+#endif
 
 #ifdef SOLARIS
 #define OSTYPE "Solaris"
@@ -79,9 +81,16 @@
 #endif 
 
 #ifdef WIN_32
+#define WINDOWS
 #define OSTYPE "Win32"
 #endif
 
+#ifdef WIN_64 /* has never been tried, just preparation */
+#define WINDOWS
+#define OSTYPE "Win64"
+#define NEWLISP64
+#endif
+
 #ifdef CYGWIN
 #define OSTYPE "Cygwin"
 #endif
@@ -90,6 +99,29 @@
 #define OSTYPE "OS/2" 
 #endif 
 
+/* include -DFFI in your makefile on the compile line 
+   and -lffi on the link line */
+#ifdef FFI
+
+#if defined(MAC_OSX)
+#include <ffi/ffi.h>
+#endif
+
+#if defined(WINDOWS) || defined(CYGWN) 
+#include "win-ffi.h" 
+#endif
+
+#if defined(LINUX) || defined(_BSD) /* makefiles specify include directory */
+#include <ffi.h>
+#endif
+
+
+#define LIBFFI " libffi"
+#else /* not FFI */
+#define LIBFFI ""
+
+#endif /* FFI */
+
 #ifdef TRU64
 #define strtoll strtol
 #define strtoull strtoul
@@ -101,6 +133,21 @@
 #define MY_RAND_MAX RAND_MAX
 #endif
 
+#ifdef LIBRARY
+#define NO_FORK
+#define NO_SPAWN
+#define NO_DEBUG
+#define NO_TIMER
+#endif
+
+#ifdef EMSCRIPTEN
+#define MY_RANDOM
+#define NO_DEBUG
+#define NO_NET_FUNCTIONS
+#define NO_WEB_FUNCTIONS
+#endif
+
+
 /* 
 This is for 64bit large file support (LFS),
 */
@@ -112,6 +159,16 @@ This is for 64bit large file support (LFS),
 #define _FILE_OFFSET_BITS 64
 #endif
 
+#ifdef WINDOWS
+/* NOTE:
+ * Windows XP [0x0501] end of support on 2014/04/08
+ * WIndows Vista [0x0600] support ends on 2017/04/11
+ */
+#ifndef _WIN32_WINNT
+#define _WIN32_WINNT 0x0600
+#endif
+#endif
+
 #include <signal.h>
 #include <errno.h>
 #include <stdio.h>
@@ -131,7 +188,7 @@ This is for 64bit large file support (LFS),
 #include <wchar.h>
 #define WCSFTIME
 #endif
-#ifdef WIN_32
+#ifdef WINDOWS
 #include <wchar.h>
 #endif
 #ifdef CYGWIN
@@ -140,10 +197,7 @@ This is for 64bit large file support (LFS),
 #endif
 #endif
 
-#ifdef WIN_32
-#ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0501 
-#endif
+#ifdef WINDOWS
 #include <windef.h>
 #include <winbase.h>
 #else
@@ -161,10 +215,9 @@ This is for 64bit large file support (LFS),
 #include <stdarg.h>
 #include <time.h>
 #include <sys/stat.h>
-#include <sys/timeb.h>
 #include <sys/types.h>
 
-#if defined(LINUX) || defined(WIN_32) || defined(OS2)
+#if defined(LINUX) || defined(WINDOWS) || defined(OS2)
 #include <malloc.h>
 #endif
 
@@ -172,10 +225,13 @@ This is for 64bit large file support (LFS),
 #include <alloca.h>
 #endif
 
-
 #ifdef OS2
 #define vasprintf my_vasprintf 
 #define MY_VASPRINTF 
+#define NO_SPAWN
+#define NO_FORK
+#define NO_SHARE
+#define NO_PACKET
 #endif
 
 #if defined(SOLARIS) || defined(TRU64) || defined(AIX)
@@ -188,19 +244,24 @@ This is for 64bit large file support (LFS),
 #define MY_SETENV 
 #endif
 
-#ifdef WIN_32
-#define LITTLE_ENDIAN
+#ifdef WINDOWS
 #define MY_VASPRINTF
 #define MY_SETENV
+#define NO_SPAWN
+#define NO_FORK
+#define NO_NET_PACKET
+#define NO_NET_PING
+
+#define LITTLE_ENDIAN
 #define LINE_FEED "\r\n"
 #define LINE_FEED_LEN 2
-
 #define getSocket(A) ((A)->_file)
-
 #define vasprintf my_vasprintf
 #define setenv my_setenv
+#ifndef MY_RANDOM
 #define random rand
 #define srandom srand
+#endif
 #define ioctl ioctlsocket
 #define off_t off64_t
 #define lseek lseek64
@@ -215,7 +276,7 @@ This is for 64bit large file support (LFS),
 
 #define realpath win32_realpath
 
-/* WIN_32 UTF16 support for file paths */
+/* WINDOWS UTF16 support for file paths */
 #ifdef SUPPORT_UTF8 
 #define USE_WIN_UTF16PATH
 #define rename rename_utf16
@@ -225,7 +286,6 @@ This is for 64bit large file support (LFS),
 #define unlink unlink_utf16
 #define chdir chdir_utf16
 #define opendir opendir_utf16
-
 #define DIR _WDIR
 #define lstat _wstat
 #define dirent _wdirent
@@ -233,15 +293,12 @@ This is for 64bit large file support (LFS),
 #define closedir _wclosedir
 #endif /* SUPPORT_UTF8 */
 
-#endif /* WIN_32 */
+#endif /* WINDOWS */
 
-#ifndef WIN_32
+#ifndef WINDOWS
 #define LINE_FEED "\n"
 #define LINE_FEED_LEN 1
-#define NET_PING
-#define NET_PACKET
 #define NANOSLEEP
-#define HAVE_FORK 
 #endif
 
 #ifndef O_BINARY
@@ -250,7 +307,14 @@ This is for 64bit large file support (LFS),
 
 #define UTF8_MAX_BYTES 6
 
+/* autosize on 32-bit ILP32 and 64-bit on LP64 and LLP64 */
+#ifndef WIN_64 /* UNIX 32 or 64 or WIN_32 */
+#define INT long
 #define UINT unsigned long 
+#else          /* WIN_64 LLP64 */
+#define INT long long
+#define UINT unsigned long long
+#endif
 
 #define INT16 short int
 #ifndef NEWLISP64
@@ -259,7 +323,7 @@ This is for 64bit large file support (LFS),
 #define MAX_LONG 0x7FFFFFFFFFFFFFFFLL
 #endif
 
-#define CONNECT_TIMEOUT 10000 /* 10 sec for connection phase  getPutPostUrl */
+#define CONNECT_TIMEOUT 10000 
 
 #ifndef NEWLISP64
 #ifdef TRU64
@@ -289,22 +353,22 @@ This is for 64bit large file support (LFS),
 #define TRUE 1
 #define FALSE 0
   
-#define MAX_STRING 2048
-#define MAX_LINE 256
-#define MAX_COMMAND_LINE 512 
-#define MAX_SYMBOL 256
-#define MAX_BIN_NO 66
+#define MAX_STRING 2048 /* buffer length */
+#define MAX_LINE 256 /* buffer length */
+/* following limits are only for parsing */
+#define MAX_COMMAND_LINE 1024 /* buffer length */
+#define MAX_SYMBOL 255 /* strlen() */
+#define MAX_DIGITS 1001 /* 1000 + sign, limitation only for parsing source */
+#define MAX_HEX_NO MAX_DIGITS /* 16 + 0x */
+#define MAX_BIN_NO MAX_DIGITS /* 64 + 0B */
+#define MAX_DECIMALS MAX_DIGITS /* 32, numbers with decimal point */
 
-#define MAX_READ_LEN 0x4000
-#define MAX_PRINT_LEN 0x4000
-#define MAX_LOAD_BUFFER 0x4000
-#define MAX_FILE_BUFFER 0x4000
+#define MAX_FILE_BUFFER 0x40000
 #define MAX_BLOCK 4095
-#define MAX_URL_LEN 256
+#define MAX_URL_LEN 255 /* strlen() */
 
 #define MAX_REGEX_EXP 16 
 
-
 /* token types */
 #define TKN_ERROR -1
 #define TKN_EMPTY 0
@@ -326,8 +390,9 @@ This is for 64bit large file support (LFS),
 #define SYMBOL_PROTECTED 0x10
 #define SYMBOL_GLOBAL 0x20
 #define SYMBOL_BUILTIN 0x40
-#define SYMBOL_PRIMITIVE 0x80
 #define SYMBOL_FFI 0x100
+#define SYMBOL_MACRO 0x200
+#define SYMBOL_DESTRUCTIVE 0x400
 
 /* cell masks */
 
@@ -339,19 +404,28 @@ This is for 64bit large file support (LFS),
 #define NUMBER_TYPE_MASK 0x0080
 #define EVAL_SELF_TYPE_MASK 0x0100
 #define INT64_MASK 0x0200
-#define CALL_CDECL_MASK 0x0400
-#define CALL_DLL_MASK 0x0800
-
+#define BIGINT_MASK 0x0400
+#define CALL_CDECL_MASK 0x1000
+#define CALL_DLL_MASK 0x2000
+#define CALL_FFI_MASK 0x4000
+#define IMPORT_MASK (CALL_CDECL_MASK | CALL_DLL_MASK | CALL_FFI_MASK)
 /* only used for type ids used in shared memory
    to indicate translation from string */
-#define SHARED_MEMORY_EVAL 0x8000
+#define SHARED_MEM_EVAL_MASK 0x8000
 
 /* cell types, do not change these without changing newlisp.c/cellCopy() */
 #define CELL_NIL (0 | EVAL_SELF_TYPE_MASK)
 #define CELL_TRUE (1 | EVAL_SELF_TYPE_MASK)
 #define CELL_INT 2 /* any INT */
 #define CELL_LONG (2 | EVAL_SELF_TYPE_MASK | NUMBER_TYPE_MASK)
+#ifndef NEWLISP64
 #define CELL_INT64 (2 | EVAL_SELF_TYPE_MASK | NUMBER_TYPE_MASK | INT64_MASK)
+#endif
+
+#ifdef BIGINT
+#define CELL_BIGINT (2 | EVAL_SELF_TYPE_MASK | NUMBER_TYPE_MASK | BIGINT_MASK)
+#endif
+
 #define CELL_FLOAT (3 | EVAL_SELF_TYPE_MASK | NUMBER_TYPE_MASK)
 #define CELL_STRING (4 | EVAL_SELF_TYPE_MASK)
 #define CELL_SYMBOL (5 | SYMBOL_TYPE_MASK)
@@ -359,26 +433,28 @@ This is for 64bit large file support (LFS),
 #define CELL_PRIMITIVE (7 | EVAL_SELF_TYPE_MASK)
 #define CELL_IMPORT_CDECL (8 | EVAL_SELF_TYPE_MASK | CALL_CDECL_MASK)
 #define CELL_IMPORT_DLL (8 | EVAL_SELF_TYPE_MASK | CALL_DLL_MASK)
-#define CELL_IMPORT_FFI (9 | EVAL_SELF_TYPE_MASK)
+#define CELL_IMPORT_FFI (9 | EVAL_SELF_TYPE_MASK | CALL_FFI_MASK)
 #define CELL_QUOTE (10 | ENVELOPE_TYPE_MASK)
 #define CELL_EXPRESSION (11 | ENVELOPE_TYPE_MASK | LIST_TYPE_MASK)
 #define CELL_LAMBDA (12 | ENVELOPE_TYPE_MASK | LIST_TYPE_MASK | EVAL_SELF_TYPE_MASK)
-#define CELL_MACRO (13 | ENVELOPE_TYPE_MASK | LIST_TYPE_MASK | EVAL_SELF_TYPE_MASK)
+#define CELL_FEXPR (13 | ENVELOPE_TYPE_MASK | LIST_TYPE_MASK | EVAL_SELF_TYPE_MASK)
 #define CELL_ARRAY (14 | ENVELOPE_TYPE_MASK | EVAL_SELF_TYPE_MASK)
 #define CELL_DYN_SYMBOL (15 | SYMBOL_TYPE_MASK)
 #define CELL_FREE 0xFF
 
+/* cell type classes */
 #define isEnvelope(A) ((A) & ENVELOPE_TYPE_MASK)
 #define isList(A) ((A) & LIST_TYPE_MASK)
-#define isArray(A) ((A) == CELL_ARRAY)
-#define isString(A) ((A) == CELL_STRING)
 #define isNumber(A) ((A) & NUMBER_TYPE_MASK)
 #define isSymbol(A) ((A) & SYMBOL_TYPE_MASK)
 #define isSelfEval(A) ((A) & EVAL_SELF_TYPE_MASK)
-#define isProtected(A) ((A) & SYMBOL_PROTECTED)
+
+/* symbol classes */
+#define isProtected(A) ((A) & (SYMBOL_PROTECTED | SYMBOL_MACRO))
 #define isBuiltin(A) ((A) & SYMBOL_BUILTIN)
 #define isGlobal(A) ((A) & SYMBOL_GLOBAL)
-#define isFFI(A) ((A) & SYMBOL_FFI)
+#define isFFIsymbol(A) ((A) & SYMBOL_FFI)
+
 #define isDigit(A) isdigit((int)(A))
 #define isHexDigit(A) isxdigit((int)(A))
 
@@ -448,7 +524,7 @@ This is for 64bit large file support (LFS),
 #define ERR_NOT_MATRIX 30
 #define ERR_WRONG_DIMENSIONS 31
 #define ERR_SINGULAR 32
-#define ERR_REGULAR_EXPRESSION 33
+#define ERR_INVALID_OPTION 33
 #define ERR_THROW_WO_CATCH 34
 #define ERR_IMPORT_LIB_NOT_FOUND 35
 #define ERR_IMPORT_FUNC_NOT_FOUND 36
@@ -486,14 +562,12 @@ This is for 64bit large file support (LFS),
 #define ERR_CANNOT_OPEN_SOCKETPAIR 68
 #define ERR_CANNOT_FORK_PROCESS 69
 #define ERR_NO_SOCKET 70
-#ifdef FFI
 #define ERR_FFI_PREP_FAILED 71
 #define ERR_FFI_INVALID_TYPE 72
 #define ERR_FFI_STRUCT_EXPECTED 73
-#define MAX_ERROR_NUMBER 73
-#else
-#define MAX_ERROR_NUMBER 70
-#endif
+#define ERR_BIGINT_NOT_ALLOWED 74
+#define ERR_CANNOT_CONVERT 75
+#define MAX_ERROR_NUMBER 75
 #define UNKNOWN_ERROR "Unknown error"
 
 /* network error handling */
@@ -558,17 +632,17 @@ extern int vasprintf (char **, const char *, va_list);
 
 typedef struct
 	{
-	int handle;
 	char *ptr;
 	char *buffer;
 	size_t position;
 	size_t size;
+	int handle;
 	} STREAM;
 
 typedef struct tagSYMBOL
 	{
-	short int flags;
-	short int color;
+	int flags;
+	int color;
 	char * name;
 	UINT contents; 
 	struct tagSYMBOL * context;
@@ -613,7 +687,6 @@ typedef struct {
 
 typedef struct
     {
-    short int type;
     char *name;
     void (*func)(void);
     ffi_cif cif;
@@ -621,6 +694,7 @@ typedef struct
     ffi_closure *clos;
     ffi_closure_data *data;
     ffi_type *cstruct;
+    short int type;
     } FFIMPORT;
 #endif
 
@@ -629,7 +703,7 @@ typedef struct
 extern char startupDir[];
 extern FILE * IOchannel;
 extern int ADDR_FAMILY;
-#ifdef WIN_32
+#ifdef WINDOWS
 extern int IOchannelIsSocket;
 #endif
 extern int MAX_CPU_STACK;
