@@ -1,7 +1,7 @@
 /*
 
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,10 +33,14 @@
 #define FIONREAD I_NREAD
 #endif
 
-#ifndef WIN_32
+#ifndef WINDOWS
 #include <sys/types.h>
+#ifndef ANDROID
+#ifndef EMSCRIPTEN
 #include <sys/ipc.h>
 #include <sys/sem.h>
+#endif
+#endif
 #include <sys/mman.h>
 #include <sys/ioctl.h>
 #endif
@@ -61,8 +65,9 @@ extern char ** environ;
 #endif
 #endif
 
-#ifdef WIN_32
+#ifdef WINDOWS
 #define fgetc win32_fgetc
+#define realpath win32_realpath
 #define random rand
 #define srandom srand
 #include <conio.h>  
@@ -77,15 +82,10 @@ See also http://www.mingw.org/MinGWiki/index.php/binary
 */
 unsigned int _CRT_fmode = _O_BINARY;
 
-
 int setenv (const char *name, const char *value, int replace);
-#ifdef USE_WIN_UTF16PATH
-INT64 fileSizeW(WCHAR * pathName);
-#endif
-
 #endif /* Win32 */
 
-#ifndef WIN_32
+#ifndef WINDOWS
 #include <sys/socket.h>
 #define SOCKET_ERROR -1
 #define INVALID_SOCKET -1
@@ -105,9 +105,11 @@ extern STREAM errorStream;
 extern UINT netErrorIdx;
 
 /* semaphore() function type */
+#ifndef NO_SEMAPHORE
 #define SEM_CREATE 0
 #define SEM_STATUS 1
 #define SEM_SIGNAL 2
+#endif
 
 /* used in fork and spawn */
 int parentPid = 0;
@@ -126,21 +128,21 @@ int flag;
 params = getString(params, &fileName);
 flag = getFlag(params);
 
-return(isFile(fileName) ? nilCell : flag ? stuffString(fileName) : trueCell);
+return(isFile(fileName, flag) ? nilCell : flag ? stuffString(fileName) : trueCell);
 }
 
-int isFile(char * fileName)
+int isFile(char * fileName, int flag)
 {
 struct stat fileInfo;
+int result;
 
-#ifdef WIN32
+#ifdef WINDOWS
 char slash;
 size_t len;
-int result;
 
 len = strlen(fileName);
 slash = *(fileName + len - 1);
-if(slash == '\\' || slash == '/')
+if((slash == '\\' || slash == '/') && (!(len >= 2 && *(fileName + len - 2) == ':')))
     *(fileName + len - 1) = 0;
 
 #ifdef USE_WIN_UTF16PATH
@@ -150,10 +152,16 @@ result = stat(fileName, &fileInfo);
 #endif
 if(slash == '\\' || slash == '/')
     *(fileName + len - 1) = slash;
-return(result);
-#else /* not WIN32 */
-return(stat(fileName, &fileInfo));
+#else /* not WINDOWS */
+result = stat(fileName, &fileInfo);
 #endif
+if(result == 0)
+    {
+    if(flag)
+        result = ! S_ISREG(fileInfo.st_mode);
+    }
+
+return(result);
 }
 
 CELL * p_isDirectory(CELL * params)
@@ -168,13 +176,13 @@ int isDir(char * fileName)
 {
 struct stat fileInfo;
 
-#ifdef WIN32
+#ifdef WINDOWS
 char slash;
 size_t len;
 
 len = strlen(fileName);
 slash = *(fileName + len - 1);
-if(slash == '\\' || slash == '/')
+if((slash == '\\' || slash == '/') && (!(len >= 2 && *(fileName + len - 2) == ':')))
     *(fileName + len - 1) = 0;
 #endif
 
@@ -184,13 +192,13 @@ if(stat_utf16(fileName, &fileInfo) != 0)
 if(stat(fileName, &fileInfo) != 0)
 #endif
     {
-#ifdef WIN32
+#ifdef WINDOWS
     *(fileName + len - 1) = slash;
 #endif
     return(0);
     }
 
-#ifdef WIN32
+#ifdef WINDOWS
 *(fileName + len - 1) = slash;
 #endif
 
@@ -252,6 +260,11 @@ if(params != nilCell)
 else 
     handle = printDevice;
 
+#ifdef WINDOWS
+/* make it work as on Unix */
+if(printDevice == 1 || printDevice == 2) handle = 0;
+#endif
+
 if(read((int)handle, &chr, 1) <= 0) return(nilCell);
 
 return(stuffInteger((UINT)chr));
@@ -263,21 +276,19 @@ CELL * p_readBuffer(CELL * params)
 UINT handle;
 size_t size, length;
 ssize_t bytesRead = 0;
-int found = 0;
 char * waitFor;
-char chr;
-STREAM stream = {0, NULL, NULL, 0, 0};
+STREAM stream = {NULL, NULL, 0, 0, 0};
 CELL * strCell;
 SYMBOL * readSptr;
+int found = 0;
+char chr;
 
 params = getInteger(params, &handle);
 params = getEvalDefault(params, &strCell);
-if(!symbolCheck)
+if(!symbolCheck || symbolCheck->contents != (UINT)strCell)
     return(errorProc(ERR_IS_NOT_REFERENCED));
 if(isProtected(symbolCheck->flags))
     return(errorProcExt2(ERR_SYMBOL_PROTECTED, stuffSymbol(symbolCheck)));
-if(symbolCheck->contents != (UINT)strCell)
-    return(errorProc(ERR_IS_NOT_REFERENCED));
 
 readSptr = symbolCheck;
 params = getInteger(params, (UINT *)&size);
@@ -321,7 +332,8 @@ if(bytesRead == 0)
     return(nilCell);
     } 
 
-stream.buffer = reallocMemory(stream.buffer, bytesRead + 1);
+if(stream.size > bytesRead)
+    stream.buffer = reallocMemory(stream.buffer, bytesRead + 1);
 readSptr->contents = (UINT)makeStringCell(stream.buffer, bytesRead);
 
 if(found) return(stuffInteger(bytesRead));
@@ -334,15 +346,18 @@ CELL * p_readFile(CELL * params)
 char * fileName;
 char * buffer = NULL;
 ssize_t size;
+#ifndef EMSCRIPTEN
 CELL * result;
+#endif
 
 params = getString(params, &fileName);
+#ifndef EMSCRIPTEN
 if(my_strnicmp(fileName, "http://", 7) == 0)
     {
     result = getPutPostDeleteUrl(fileName, params, HTTP_GET, CONNECT_TIMEOUT);
     return((my_strnicmp((char *)result->contents, (char *)"ERR:", 4) == 0) && netErrorIdx ? nilCell : result);
     }
-
+#endif
 if((size = readFile(fileName, &buffer)) == -1)
     return(nilCell);
 
@@ -353,7 +368,7 @@ return(makeStringCell(buffer, size));
 ssize_t readFile(char * fileName, char * * buffer)
 {
 int handle; 
-size_t size;
+off_t size;
 struct stat fileInfo;
 
 fileName = getLocalPath(fileName);
@@ -451,16 +466,20 @@ CELL * appendWriteFile(CELL * params, char * type)
 char * fileName;
 char * buffer;
 size_t size;
+#ifndef EMSCRIPTEN
 CELL * result;
+#endif
 
 params = getString(params, &fileName);
 
+#ifndef EMSCRIPTEN
 if(my_strnicmp(fileName, "http://", 7) == 0)
     {
     result = getPutPostDeleteUrl(fileName, params, 
                 (*type == 'w') ? HTTP_PUT : HTTP_PUT_APPEND, CONNECT_TIMEOUT);
     return((my_strnicmp((char *)result->contents, (char *)"ERR:", 4) == 0) && netErrorIdx ? nilCell : result);
     }
+#endif
 
 getStringSize(params, &buffer, &size, TRUE);
 
@@ -571,7 +590,7 @@ if(params == nilCell)
 else
     {
 #ifdef LFS
-    getInteger64(params, &paramPosition);
+    getInteger64Ext(params, &paramPosition, TRUE);
 #else
     getInteger(params, (UINT *)&paramPosition);
 #endif
@@ -599,7 +618,12 @@ return(stuffInteger(paramPosition));
 
 char * readStreamLine(STREAM * stream, FILE * inStream)
 {
+#ifdef OLD_READ_STREAM /* pre 10.5.8 */
 int chr;
+#else
+char buff[MAX_STRING];
+size_t l;
+#endif
 
 openStrStream(stream, MAX_STRING, 1);
 
@@ -607,6 +631,7 @@ openStrStream(stream, MAX_STRING, 1);
 do {
 errno = 0;
 #endif
+#ifdef TRUE64 /* pre 10.5.8 also all other OS */
 while((chr = fgetc(inStream)) != EOF)
     {
     if(chr == '\n') break;
@@ -617,12 +642,34 @@ while((chr = fgetc(inStream)) != EOF)
         }
     writeStreamChar(stream, chr);
     }
+#else
+while(fgets(buff, MAX_STRING, inStream) != NULL)
+    {
+    l=strlen(buff);
+    if(buff[l-1] == 0x0A)
+        {
+        buff[--l] = 0;
+        if(buff[l-1] == 0x0D)
+            buff[--l] = 0;
+        writeStreamStr(stream, buff, l);
+        break;
+        }
+    writeStreamStr(stream, buff, l);
+    }
+#endif /* pre 10.5.8 also all other OS */
 #ifdef TRU64
 } while (errno == EINTR);
 #endif
 
-if(feof(inStream)) clearerr(inStream);
+#ifdef TRU64 /* and pre 10.5.8 on all other OS */
 if(chr == EOF && stream->position == 0) return(NULL);
+#else
+if(feof(inStream)) 
+    {
+    clearerr(inStream);
+    if(stream->position == 0) return(NULL);
+    }
+#endif
 return(stream->buffer);
 }
 
@@ -640,6 +687,11 @@ if(params != nilCell)
 else 
     handle = printDevice;
 
+#ifdef WINDOWSXXX
+/* make it work as on Unix */
+if(printDevice == 1 || printDevice == 2) handle = 0;
+#endif
+
 /* check if stream input can be done */
 fstream = (handle == 0) ? IOchannel : getIOstream(handle);
 if(fstream != NULL)
@@ -649,8 +701,8 @@ if(fstream != NULL)
     return(stuffString(line));
     }
 
-/* do raw handle input only happens when using read-line on sockets on UNIX
-   and pipes on Windows  */
+/* do raw handle input, only happens when using read-line on 
+   sockets on UNIX and pipes on Windows  */
 openStrStream(&readLineStream, MAX_STRING, 1);
 while(TRUE)
     {
@@ -682,7 +734,7 @@ char * getLocalPath(char * fileName)
 if(my_strnicmp(fileName, "file://", 7) == 0)
     fileName = fileName + 7;
 
-#ifdef WIN_32
+#ifdef WINDOWS
 if(*fileName == '/' && *(fileName + 2) == ':')
     fileName = fileName + 1;
 #endif
@@ -694,10 +746,13 @@ return(fileName);
 int openFile(char * fileName, char * accessMode, char * option)
 {
 int blocking = 0;
+#ifndef WINDOWS
+int handle;
+#endif
 
 fileName = getLocalPath(fileName);
 
-#ifndef WIN_32
+#ifndef WINDOWS
 if(option != NULL && *option == 'n')
     blocking = O_NONBLOCK;
 #endif
@@ -706,7 +761,7 @@ if(*accessMode == 'r')
     return(open(fileName, O_RDONLY | O_BINARY | blocking, 0));
 
 else if(*accessMode == 'w')
-#ifdef WIN_32
+#ifdef WINDOWS
     return(open( fileName, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, S_IREAD | S_IWRITE) );
 #else
     return(open(fileName,O_WRONLY | O_CREAT | O_TRUNC | O_BINARY | blocking, 
@@ -718,11 +773,20 @@ else if(*accessMode == 'u')
 
 else if(*accessMode == 'a')
        {
-#ifdef WIN_32
+#ifdef WINDOWS
        return(open(fileName, O_RDWR | O_APPEND | O_BINARY | O_CREAT, S_IREAD | S_IWRITE));
 #else
-       return(open(fileName, O_RDWR | O_APPEND | O_BINARY | O_CREAT,
-          S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IWGRP | S_IWOTH)); /* rw-rw-rw */
+       handle = open(fileName, O_RDWR | O_APPEND | O_BINARY | O_CREAT,
+          S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IWGRP | S_IWOTH); /* rw-rw-rw */
+#ifdef EMSCRIPTEN
+       /* oppen append is broken on Emscripten, open for update but filepointer
+          stays at the beginning and old contents is overwritten */
+       if(lseek(handle, 0, SEEK_END) != -1)
+                return(handle);
+#else
+       return(handle);
+#endif
+
 #endif
        }
 
@@ -779,14 +843,18 @@ return(rename(oldName, newName) == 0 ? trueCell : nilCell);
 CELL * p_deleteFile(CELL * params)
 {
 char * fileName;
+#ifndef EMSCRIPTEN
 CELL * result;
+#endif
 
 params = getString(params, &fileName);
+#ifndef EMSCRIPTEN
 if(my_strnicmp(fileName, "http://", 7) == 0)
     {
     result = getPutPostDeleteUrl(fileName, params, HTTP_DELETE, CONNECT_TIMEOUT);
     return((my_strnicmp((char *)result->contents, (char *)"ERR:", 4) == 0) && netErrorIdx ? nilCell : result);
     }
+#endif
 
 fileName = getLocalPath(fileName);
 return(unlink(fileName) == 0 ? trueCell : nilCell);
@@ -796,9 +864,7 @@ return(unlink(fileName) == 0 ? trueCell : nilCell);
 CELL * p_makeDir(CELL * params)
 {
 char * dirString;
-UINT mode;
-
-mode = 0777; /* drwxrwxrwx  gets user masked to drwxr-xr-x on most UNIX */
+UINT mode = 0777; /* drwxrwxrwx  gets user masked to drwxr-xr-x on most UNIX */
 
 /* consume param regardless of OS */
 params = getString(params, &dirString);
@@ -809,7 +875,7 @@ if(params != nilCell)
     mode = mode > 0xfff ? 0xfff : mode;
     }
 
-#ifdef WIN_32
+#ifdef WINDOWS
 return(mkdir(dirString) == 0 ? trueCell : nilCell);
 #else
 return(mkdir(dirString, (mode_t)mode) == 0 ? trueCell : nilCell);
@@ -851,7 +917,8 @@ if(params != nilCell)
         {
         params = getString(params, &pattern);
         if(params != nilCell)
-            getInteger(params, &options);
+            /* 10.6.1 also accept string for options */
+            parseRegexOptions(params, (UINT *)&options, TRUE);
         }
     }
 else dirPath = ".";
@@ -882,18 +949,31 @@ return(dirList);
 
 #define DOT_PATH ".\000"
 
+
 CELL * p_realpath(CELL * params)
 {
 char  path[PATH_MAX];
-char * dir;
+char * name;
 
 if(params != nilCell)
-    getString(params, &dir);
-else dir = DOT_PATH;
+    {
+    params = getString(params, &name);
+    if(getFlag(params))
+        {
+        if((name = which(name, alloca(PATH_MAX))) == NULL)
+            return(nilCell);
+        return(stuffString(name));
+        }
+    }
+else name = DOT_PATH;
 
-if(realpath(dir, path) == NULL)
+if(realpath(name, path) == NULL)
     return(nilCell);
 
+#ifdef _BSD /* behaves like Windows */
+if(isFile(path, 0)) return(nilCell);
+#endif
+
 return(stuffString(path));
 }
 
@@ -906,7 +986,7 @@ int result = 0;
 
 params = getString(params, &pathName);
 
-#ifdef WIN_32 /* has no link-flag */
+#ifdef WINDOWS /* has no link-flag */
 #ifdef USE_WIN_UTF16PATH
 result = stat_utf16(pathName, &fileInfo);
 #else
@@ -938,11 +1018,7 @@ list = stuffIntegerList(
 #ifndef NEWLISP64
 #ifdef LFS
 ((CELL *)list->contents)->type = CELL_INT64;
-#ifdef USE_WIN_UTF16PATH
-*(INT64 *)&((CELL *)list->contents)->aux = fileSize_utf16(pathName);
-#else
-*(INT64 *)&((CELL *)list->contents)->aux = fileSize(pathName);
-#endif /* UTF16PATH */
+*(INT64 *)&((CELL *)list->contents)->aux = (INT64)fileInfo.st_size;
 #endif /* LFS */
 #endif /* NEWLISP64 */
 
@@ -958,26 +1034,32 @@ return(list);
 
 #ifdef LFS
 INT64 fileSize(char * pathName)
-{
-int handle;
-INT64 size;
-
-#ifndef WIN_32
-handle = open(pathName,O_RDONLY | O_BINARY | O_NONBLOCK , 0);
 #else
-handle = open(pathName,O_RDONLY | O_BINARY, 0);
+size_t fileSize(char * pathName)
 #endif
-size = lseek(handle, 0, SEEK_END);
-close(handle);
-if(size == -1) size = 0;
-return(size);
+{
+struct stat fileInfo;
+int result;
+
+#ifdef WINDOWS /* has no link-flag */
+#ifdef USE_WIN_UTF16PATH
+result = stat_utf16(pathName, &fileInfo);
+#else
+result = stat(pathName, &fileInfo);
+#endif
+#else /* Unix */
+result = stat(pathName, &fileInfo);
+#endif
+
+if(result != 0) return 0;
+
+return(fileInfo.st_size);
 }
-#endif
 
 
 /* ------------------------- processes and pipes ------------------------- */
 
-#ifndef WIN_32
+#ifndef WINDOWS
 CELL * p_system(CELL *params)
 {
 char * command;
@@ -1090,8 +1172,8 @@ return(argc);
 }
 
 
-
-#ifdef WIN_32
+#ifndef EMSCRIPTEN
+#ifdef WINDOWS
 int kill(pid_t pid, int sig);
 int winPipe(UINT * inpipe, UINT * outpipe);
 UINT winPipedProcess(char * command, int inpipe, int outpipe, int option);
@@ -1139,7 +1221,7 @@ return(stuffInteger(result));
 }
 
 
-#else /* not WIN_32 */
+#else /* not WINDOWS */
 
 CELL * p_pipe(CELL * params)
 {
@@ -1231,6 +1313,7 @@ freeMemory(cmd);
 return(stuffInteger(forkResult));
 }
 
+#ifndef NO_FORK
 CELL * p_fork(CELL * params)
 {
 int forkResult;
@@ -1247,7 +1330,7 @@ if(forkResult == 0)
 
 return(stuffInteger(forkResult));
 }
-
+#endif
 
 /* ------------------------------------------------------------------------- */
 
@@ -1268,8 +1351,7 @@ int thisSocket = 0;
 fd_set myFdSet;             /* set of all child sockets */
 
 
-#ifdef HAVE_FORK
-#ifndef OS2
+#ifndef NO_SPAWN
 
 typedef struct 
     {
@@ -1368,8 +1450,7 @@ while(pidSpawn)
             }
         else if(mode == PROCESS_SPAWN_ABORT)
             {
-            /* close(pidSpawn->socket);  10.3.6 */
-            FD_CLR(pidSpawn->socket, &myFdSet); /* 10.3.6 */
+            FD_CLR(pidSpawn->socket, &myFdSet);
             kill(pidSpawn->pid, 9);
             waitpid(pidSpawn->pid, (int *)0, 0);
             }
@@ -1380,7 +1461,9 @@ while(pidSpawn)
             snprintf(str, 32, "%s %d", ABEND, result);
             sPtr->contents = (UINT)stuffString(str);
             }
-            
+           
+	/* close parent socket */
+        if(pidSpawn->socket) close(pidSpawn->socket); 
         checkDeleteShareFile(pidSpawn->result_addr);
         /* unmap shared result memory */
         munmap(pidSpawn->result_addr, pagesize);
@@ -1392,7 +1475,6 @@ while(pidSpawn)
     }
 }
 
-
 /* spawn (fork) a process and assign result to the symbol given
      (spawn <quoted-symbol> <epxression>) => pid
    creates a memory share and passes it to the spawned process
@@ -1412,14 +1494,11 @@ void * address; /* share memory area for result */
 SYMBOL * symPtr;
 int sockets[2] = {0, 0};
 
-/* make signals processable by waitpid() in p_sync() */
-signal(SIGCHLD, SIG_DFL);
-
 if((address = mmap( 0, pagesize, 
     PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0)) == (void*)-1)
         return(nilCell);
 
-memset(address, 0, sizeof(long));
+memset(address, 0, sizeof(INT));
 
 params = getSymbol(params, &symPtr);
 if(isProtected(symPtr->flags))
@@ -1434,8 +1513,8 @@ if(getFlag(params->next))
     {
     if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockets) == -1) 
         {
-        purgeSpawnList(TRUE);
-        errorProc(ERR_CANNOT_OPEN_SOCKETPAIR); 
+        munmap(address, pagesize);
+        return(errorProc(ERR_CANNOT_OPEN_SOCKETPAIR)); 
         }
 
     /* add the parent socket to myFdSet */ 
@@ -1444,8 +1523,13 @@ if(getFlag(params->next))
     FD_SET(sockets[0], &myFdSet);
     }
 
+/* make signals processable by waitpid() in p_sync() */
+signal(SIGCHLD, SIG_DFL);
+
 if((forkPid = fork()) == -1)
     {
+    if(sockets[0]) close(sockets[0]);
+    if(sockets[1]) close(sockets[1]);
     munmap(address, pagesize);
     return(nilCell);
     }
@@ -1462,6 +1546,8 @@ if(forkPid == 0) /* the child process */
     purgeSpawnList(FALSE);
     /* evaluate and write result to shared memory */
     readWriteShared(address, params, TRUE);
+    /* close child socket */
+    if(thisSocket) close(thisSocket);
     exit(0);
     }
 
@@ -1592,7 +1678,7 @@ fd_set thisFdSet;
 tv.tv_sec = 0;
 tv.tv_usec = 892 + random() / 10000000;
 
-#if defined(SUNOS) || defined(LINUX) || defined(CYGWIN)
+#if defined(SUNOS) || defined(LINUX) || defined(CYGWIN) || defined(AIX) 
 memcpy(&thisFdSet, &myFdSet, sizeof(fd_set));
 #else
 FD_COPY(&myFdSet, &thisFdSet);
@@ -1606,10 +1692,7 @@ else /* SELECT_WRITE_READY */
 if(ready == 0) return(pidList);
 
 if(ready < 0) 
-    {
-    printf("select failed\n");
     return(pidList);
-    }
 
 child = mySpawnList;
 while (child != NULL)
@@ -1724,7 +1807,7 @@ CELL * readWriteSocket(int socket, CELL * params)
 {
 char * buffer;
 CELL * cell;
-STREAM strStream = {0, NULL, NULL, 0, 0};
+STREAM strStream = {NULL, NULL, 0, 0, 0};
 UINT length;
 ssize_t size, bytesReceived;
 struct timeval tv;
@@ -1802,9 +1885,7 @@ if(ready == 1)
 return(nilCell);
 }
 
-#endif /* not OS 2 */
-
-#endif /* HAVE_FORK */
+#endif /* NO_SPAWN */
 
 /* --------------------------- end Cilk ------------------------------------- */
 
@@ -1856,8 +1937,8 @@ return(trueCell);
 }
 
 /* ------------------------------ semaphores --------------------------------- */
-
-#ifdef WIN_32
+#ifndef NO_SEMAPHORE
+#ifdef WINDOWS
 
 UINT winCreateSemaphore(void);
 UINT winWaitSemaphore(UINT hSemaphore);
@@ -1868,7 +1949,7 @@ int getSemaphoreCount(UINT hSemaphore);
 CELL * p_semaphore(CELL * params)
 {
 UINT sem_id;
-long value;
+INT value;
 
 if(params != nilCell)
     {
@@ -1907,14 +1988,11 @@ if(params != nilCell)
 if((sem_id = winCreateSemaphore()) == 0) return(nilCell);
 return(stuffInteger(sem_id));
 }
-#endif
-
-#ifndef WIN_32
-/* Mac OS X, Linux/UNIX */
+#else /* Mac OS X, Linux/UNIX */
 
 CELL * p_semaphore(CELL * params)
 {
-long sem, value, result;
+INT sem, value, result;
 
 if(params == nilCell)
     {
@@ -2020,15 +2098,17 @@ if(semctl(sem_id, 0, SETVAL, 0) == -1) /* LINUX, BSD, TRU64 */
 return(sem_id);
 }
 
-#endif  /* Mac OS X, Linux, UNIX - not WIN_32 */
+#endif /* MAC OSX, Unix, Linux */
+#endif /* NO_SEMAPHORE */
 
-#ifdef WIN_32
+
+#ifndef NO_SHARE
+
+#ifdef WINDOWS
 UINT winSharedMemory(int size);
 UINT * winMapView(UINT handle, int size);
 #endif
 
-#ifndef OS2
-
 /* since 10.1.0 also can share object > pagesize
    in this case transfer is done using /tmp/nls-*
    ot /temp/nls-* files (Win32)
@@ -2038,7 +2118,7 @@ CELL * p_share(CELL * params)
 {
 void * address;
 CELL * cell;
-#ifdef WIN_32
+#ifdef WINDOWS
 UINT handle;
 #endif
 
@@ -2046,7 +2126,7 @@ UINT handle;
 if(params != nilCell) 
     {
     cell = evaluateExpression(params);
-#ifndef WIN_32
+#ifndef WINDOWS
     if(isNil(cell)) /* release shared address */
         {
         getInteger(params->next, (UINT *)&address);
@@ -2059,7 +2139,7 @@ if(params != nilCell)
 #endif
     getIntegerExt(cell, (UINT *)&address, FALSE);
     params = params->next;
-#ifdef WIN_32
+#ifdef WINDOWS
     if((address = winMapView((UINT)address, pagesize)) == NULL)
         return(nilCell);
 #endif
@@ -2067,7 +2147,7 @@ if(params != nilCell)
     }
 
 /* get shared memory UNIX */
-#ifndef WIN_32
+#ifndef WINDOWS
 if((address = (UINT*)mmap(
     0, pagesize, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0)) == (void*)-1)
         return(nilCell);
@@ -2075,7 +2155,7 @@ if((address = (UINT*)mmap(
 memset((char *)address, 0, pagesize);
 return(stuffInteger((UINT)address));
 
-/* get shared memory WIN_32 */
+/* get shared memory WINDOWS */
 #else 
 
 if((handle = winSharedMemory(pagesize)) == 0)
@@ -2088,12 +2168,12 @@ memset((char *)address, 0, pagesize);
 return(stuffInteger(handle));
 #endif
 }
-#endif /* no OS2 */
+#endif /* NO_SHARE  */
 
-/* evaluate the expression in params ans the write the result
+/* evaluate the expression in params and the write the result
    to shared memory. If size > pagesize use files tmp files
    for transfer. For atomic datatypes are xlation into message
-   id ptimized for speed (but doesn't bring much in overall
+   optimized for speed (but doesn't bring much in overall
    performance, should perhaps be taken out and only use
    readWriteSharedExpression() */
 CELL * readWriteShared(UINT * address, CELL * params, int flag)
@@ -2106,7 +2186,7 @@ int errNo;
 /* write to shared memory */
 if(params != nilCell) 
     {
-    if(flag)
+    if(flag) /* in spawned process */
         {
         if((cell = evaluateExpressionSafe(params, &errNo)) == NULL)
             cell = stuffString(errorStream.buffer);
@@ -2117,10 +2197,10 @@ if(params != nilCell)
     /* if a previous share mem file is still present, delete it
        when *address == 0 when called from Cilk then file is
        deleted p_message(). Here only used from p_share() */
-    if(*address == (CELL_STRING | SHARED_MEMORY_EVAL))
+    if(*address == (CELL_STRING | SHARED_MEM_EVAL_MASK))
         checkDeleteShareFile(address);
 
-    /* write list types */
+    /* write anything not bool, number or string */
     if((cell->type & COMPARE_TYPE_MASK) > (CELL_STRING & COMPARE_TYPE_MASK))
         return(copyCell(readWriteSharedExpression(address, cell)));
 
@@ -2128,18 +2208,18 @@ if(params != nilCell)
         {
         case CELL_NIL:
             *address = cell->type;
-#ifdef WIN_32
+#ifdef WINDOWS
             UnmapViewOfFile(address);
 #endif
             return(nilCell);
         case CELL_TRUE:
             *address = cell->type;
-#ifdef WIN_32
+#ifdef WINDOWS
             UnmapViewOfFile(address);
 #endif
             return(trueCell);
         case CELL_LONG:
-            *(address + 1) = sizeof(long);
+            *(address + 1) = sizeof(INT);
             *(address + 2) = cell->contents;
             break;
 #ifndef NEWLISP64
@@ -2160,19 +2240,19 @@ if(params != nilCell)
 #endif /* NEWLISP64 */
         case CELL_STRING:
             getStringSize(cell, &str, &size, FALSE);
-            if(size > (pagesize - 3 * sizeof(long)))
+            if(size > (pagesize - 3 * sizeof(INT)))
                 return(copyCell(readWriteSharedExpression(address, cell)));
 
             *(address + 1) = size;
             memcpy((char *)(address + 2), str, size);
-            *((char *)address + 2 * sizeof(long) + size) = 0;
+            *((char *)address + 2 * sizeof(INT) + size) = 0;
             break;
         default:
             return(errorProcExt(ERR_ILLEGAL_TYPE, cell));
         }
 
     *address = cell->type;
-#ifdef WIN_32
+#ifdef WINDOWS
     UnmapViewOfFile(address);
 #endif
     return(copyCell(cell));
@@ -2182,12 +2262,12 @@ if(params != nilCell)
 switch(*address & RAW_TYPE_MASK)
     {
     case CELL_NIL:
-#ifdef WIN_32 
+#ifdef WINDOWS 
         UnmapViewOfFile(address);
 #endif
         return(nilCell);
     case CELL_TRUE:
-#ifdef WIN_32 
+#ifdef WINDOWS 
         UnmapViewOfFile(address);
 #endif
         return(trueCell);   
@@ -2210,7 +2290,7 @@ switch(*address & RAW_TYPE_MASK)
 #endif
         break;
     case CELL_STRING:
-        if(*address & SHARED_MEMORY_EVAL)
+        if(*address & SHARED_MEM_EVAL_MASK)
             return(readWriteSharedExpression(address, nilCell));
         size = *(address + 1);
         cell = makeStringCell(allocMemory(size + 1), size);
@@ -2220,7 +2300,7 @@ switch(*address & RAW_TYPE_MASK)
         return(nilCell);
     }
 
-#ifdef WIN_32
+#ifdef WINDOWS
         UnmapViewOfFile(address);
 #endif
 return(cell);
@@ -2236,7 +2316,7 @@ return(cell);
 CELL * readWriteSharedExpression(UINT * address, CELL * params)
 {
 ssize_t size;
-STREAM strStream = {0, NULL, NULL, 0, 0};
+STREAM strStream = {NULL, NULL, 0, 0, 0};
 CELL * cell;
 char * buffer = NULL;
 /* int errNo; */
@@ -2245,7 +2325,7 @@ char * buffer = NULL;
 if(params == nilCell)
     {
     size = *(address + 1);
-    if(size < (pagesize - 3 * sizeof(long) ))
+    if(size < (pagesize - 3 * sizeof(INT) ))
         {   
         cell = sysEvalString((char *)(address + 2), 
                 currentContext, nilCell, READ_EXPR_SYNC);
@@ -2270,19 +2350,24 @@ prettyPrintFlags &= ~PRETTYPRINT_STRING;
 
 *(address + 1) = strStream.position;
 
-if(strStream.position < pagesize - 3 * sizeof(long))
+if(strStream.position < pagesize - 3 * sizeof(INT))
     {
     memcpy((char *)(address + 2), strStream.buffer, strStream.position);
-    *((char *)address + 2 * sizeof(long) + strStream.position) = 0;
+    *((char *)address + 2 * sizeof(INT) + strStream.position) = 0;
     }
 else
     {
     checkDeleteShareFile(address);
-    memset((char *)(address + 2), 0, pagesize - 2 * sizeof(long));
-#ifndef WIN_32
+    memset((char *)(address + 2), 0, pagesize - 2 * sizeof(INT));
+#ifndef WINDOWS
+#ifdef ANDROID
+    strncpy((char *)(address + 2), "/data/tmp/nls-", 15);
+    getUUID((char *)(address + 2) + 14, 0);
+#else /* all other UNIX */
     strncpy((char *)(address + 2), "/tmp/nls-", 10);
     getUUID((char *)(address + 2) + 9, 0);
-#else
+#endif
+#else /* WINDOWS */
     strncpy((char *)(address + 2), "/temp/nls-", 11);
     getUUID((char *)(address + 2) + 10, 0);
 #endif
@@ -2290,15 +2375,19 @@ else
     }
 closeStrStream(&strStream);
 
-*address = (CELL_STRING | SHARED_MEMORY_EVAL);
+*address = (CELL_STRING | SHARED_MEM_EVAL_MASK);
 return(cell);
 }
 
 void checkDeleteShareFile(UINT * address)
 {
-if(     (*address == (CELL_STRING | SHARED_MEMORY_EVAL)) &&
-#ifndef WIN_32
+if(     (*address == (CELL_STRING | SHARED_MEM_EVAL_MASK)) &&
+#ifndef WINDOWS
+#ifdef ANDROID
+        (strncmp((char *)(address + 2), "/data/tmp/nls-", 9) == 0) &&
+#else
         (strncmp((char *)(address + 2), "/tmp/nls-", 9) == 0) &&
+#endif
         (strlen((char *)(address + 2)) == 45) )
 #else
         (strncmp((char *)(address + 2), "/temp/nls-", 10) == 0) &&
@@ -2306,6 +2395,7 @@ if(     (*address == (CELL_STRING | SHARED_MEMORY_EVAL)) &&
 #endif
     unlink((char *)(address + 2)); 
 }
+#endif /* ifndef EMSCRIPTEN */
 
 extern int ADDR_FAMILY;
 CELL * p_systemInfo(CELL * params)
@@ -2367,7 +2457,8 @@ struct tm * ltm;
 char * ct;
 char * fmt;
 ssize_t offset;
-time_t tme;
+/* time_t tme; 10.6.1. */
+UINT tme;
 size_t size;
 
 #ifdef SUPPORT_UTF8
@@ -2387,8 +2478,9 @@ if(params == nilCell)
     }
 else
     {
-    params = getInteger(params, (UINT *)&tme);
-    t = tme;
+    /* 10.6.1 */
+    params = getInteger(params, &tme);
+    t = (time_t)tme;
     
     if(params != nilCell)
         {
@@ -2437,10 +2529,13 @@ INT64 microSecTime(void)
 {
 struct timeval tv;
 struct tm * ttm;
+time_t sec;
 
 gettimeofday(&tv, NULL);
 
-ttm = localtime((time_t *)&tv.tv_sec);
+/* ttm = localtime((time_t *)&tv.tv_sec); 10.6.1 */
+sec = tv.tv_sec;
+ttm = localtime(&sec);
 
 return (ttm->tm_hour * 3600000000LL + 
        ttm->tm_min * 60000000LL + ttm->tm_sec * 1000000 + 
@@ -2484,7 +2579,7 @@ usec = (UINT64)1000000 * out.tv_sec + out.tv_usec;
 return(usec);
 }
 
-#ifndef WIN_32
+#ifndef WINDOWS
 CELL * p_dateParse(CELL * params)
 {
 struct tm ttm;
@@ -2522,7 +2617,7 @@ double diff;
 
 gettimeofday(&start, NULL);
 if(params->next != nilCell)
-    getInteger64(params->next, &N);
+    getInteger64Ext(params->next, &N, TRUE);
 
 resultIdxSave = resultStackIdx;
 while(N--)  
@@ -2534,14 +2629,14 @@ while(N--)
 gettimeofday(&end, NULL);
 
 diff = (1.0 * timediff64_us(end, start)) / 1000;
-return(stuffFloat(&diff));
+return(stuffFloat(diff));
 }
 
 
 CELL * p_timeOfDay(CELL * params)
 {
 double microSecs = microSecTime()/1000.0;
-return(stuffFloat(&microSecs));
+return(stuffFloat(microSecs));
 }
 
 
@@ -2549,21 +2644,24 @@ CELL * p_now(CELL * params)
 {
 struct timeval tv;
 struct tm *ttm;
-#ifndef WIN_32
+#ifndef WINDOWS
 struct tm *ltm;
 #ifndef SUNOS
 #ifndef OS2
-long gmtoff;
+#ifndef AIX
+INT gmtoff;
 UINT isdst;
 #endif
 #endif
-#else /* WIN_32 */
+#endif
+#else /* WINDOWS */
 TIME_ZONE_INFORMATION timeZone;
 #endif
 ssize_t offset = 0;
+time_t sec;
 CELL * cell;
 
-gettimeofday(&tv, NULL); /* get secs and microsecs */
+gettimeofday(&tv, NULL);
 
 if(params != nilCell)
     {
@@ -2572,10 +2670,11 @@ if(params != nilCell)
         tv.tv_sec += offset;
     }
 
-#ifndef WIN_32
+#ifndef WINDOWS
 ltm = localtime((time_t *)&tv.tv_sec);
 #ifndef SUNOS
 #ifndef OS2
+#ifndef AIX
 isdst = ltm->tm_isdst;
 
 #ifdef CYGWIN
@@ -2586,11 +2685,14 @@ gmtoff = ltm->tm_gmtoff/60;
 
 #endif
 #endif
-#else /* WIN_32 */
+#endif
+#else /* WINDOWS */
 GetTimeZoneInformation(&timeZone);
 #endif
 
-ttm = gmtime((time_t *)&tv.tv_sec);
+/* ttm = gmtime((time_t *)&tv.tv_sec); 10.6.1 */
+sec = tv.tv_sec;
+ttm = gmtime(&sec);
 
 cell = stuffIntegerList(
     11,
@@ -2620,7 +2722,7 @@ cell = stuffIntegerList(
 #endif
 #endif
 
-#if defined(WIN_32)
+#if defined(WINDOWS)
      -timeZone.Bias,
      timeZone.DaylightBias  
 #endif
@@ -2640,10 +2742,13 @@ CELL * p_dateList(CELL * params)
 {
 struct tm *ttm;
 ssize_t timeValue;
+time_t timer;
 CELL * cell;
 
 params = getInteger(params, (UINT*)&timeValue);
-ttm = gmtime((time_t *)&timeValue);
+/* ttm = gmtime((time_t *)&timeValue); 10.6.1 */
+timer = (time_t)timeValue;
+ttm = gmtime(&timer);
 
 cell = stuffIntegerList(
     8,
@@ -2683,7 +2788,7 @@ params = getInteger(params, (UINT *)&year);
 params = getInteger(params, (UINT *)&month);
 params = getInteger(params, (UINT *)&day);
 
-if(year < 1970) return(stuffInteger(0));
+/* if(year < 1970) return(stuffInteger(0)); */
 
 hour = min = sec = 0;
 if(params != nilCell)
@@ -2702,20 +2807,27 @@ return(stuffInteger((UINT)dateValue));
 #endif
 }
 
-
+/* changed for 10.6.1 where time_t can be 64-bit on 32-bit Windows */
 time_t calcDateValue(int year, int month, int day, int hour, int min, int sec)
 {
 time_t dateValue;
+INT64 value;
 
-dateValue = 367 * year - (7 * (year + ((month + 9) / 12)))/4 
+value = 367 * year - (7 * (year + ((month + 9) / 12)))/4 
             + (275 * month)/9 + day + 1721013;
 
-dateValue = dateValue * 24 * 3600 + hour * 3600 + min * 60 + sec 
+value = value * 24 * 3600 + hour * 3600 + min * 60 + sec 
             - 413319296; /* correction for 1970-1-1 */
 
-#ifdef NEWLISP64
-dateValue = dateValue & 0xffffffff;
-#endif
+if(sizeof(time_t) == 8)
+    {
+    if(value & 0x80000000)
+        dateValue = value | 0xFFFFFFFF00000000LL;
+    else
+        dateValue = value & 0x00000000FFFFFFFF; 
+    }
+else
+    dateValue = value;
 
 return(dateValue);
 }
@@ -2736,7 +2848,7 @@ nanosleep(&tm, 0);
 
 #else
 
-#ifdef WIN_32
+#ifdef WINDOWS
 Sleep(ms);
 #else
 sleep((ms + 500)/1000);
@@ -2772,7 +2884,7 @@ nanoSecsInt = (milliSecsFloat - (int)milliSecsFloat) * 1000000;
 if(nanoSecsInt) myNanoSleep(nanoSecsInt);
 #endif
 
-return(stuffFloat(&milliSecsFloat));
+return(stuffFloat(milliSecsFloat));
 }
 
 /* -------------------------------- environment functions ------------------- */
@@ -2877,7 +2989,7 @@ return(envList);
 /* thanks to Peter van Eerten for contributing this function */
 CELL * p_readKey(CELL * params)
 {
-#if defined(WIN_32) || defined(OS2)
+#if defined(WINDOWS) || defined(OS2)
 return(stuffInteger(getch()));
 #else
 
@@ -2911,7 +3023,7 @@ return(stuffInteger(c));
 
 /* --------------------- peek a file descriptor ------------------------------*/
 
-#ifndef WIN_32
+#ifndef WINDOWS
 CELL * p_peek(CELL * params)
 {
 UINT handle;
@@ -2963,6 +3075,27 @@ return((int)pSize);
 
 #endif
 
+
+#ifdef MY_RANDOM /* used with #define EMSCRIPTEN */
+
+int  m_w = 0x12345678;    /* must not be zero, nor 0x464fffff */
+int  m_z = 0x23456789;    /* must not be zero, nor 0x9068ffff */
+ 
+long int random(void)
+{
+m_z = 36969 * (m_z & 65535) + (m_z >> 16);
+m_w = 18000 * (m_w & 65535) + (m_w >> 16);
+return (((m_z << 16) + m_w)  & 0x7fffffff);  /* 31-bit result */
+}
+
+void srandom(unsigned int init)
+{
+m_z = init;
+m_w = 3 * init;
+}
+
+#endif
+
 /* ---------------------- Universal Unique ID version 1 and 3 ----------- */
 
 #define UINT16 unsigned short
@@ -2997,7 +3130,7 @@ gettimeofday(&tp, (struct timezone *)0);
 /* add UUID UTC offset Oct 15, 1582 */
 timestamp = tp.tv_sec * (INT64)10000000 + tp.tv_usec * 10 + OCT151582; 
 
-#ifdef WIN_32
+#ifdef WINDOWS
 if(timestamp <= last_time) timestamp = last_time + 1;
 #else
 if(timestamp < last_time) clock_seq++;
@@ -3037,7 +3170,7 @@ if(uuid_version == 4)
     {
     clock_seq = random();
     uuid.time_low = random();
-#ifdef WIN_32
+#ifdef WINDOWS
     uuid.time_low |= (random() << 16);
 #endif
     uuid.time_mid = random();
@@ -3045,7 +3178,7 @@ if(uuid_version == 4)
     }
 else
     {
-    uuid.time_low = (unsigned long)(timestamp & 0xFFFFFFFF);
+    uuid.time_low = (unsigned int)(timestamp & 0xFFFFFFFF);
     uuid.time_mid = (unsigned short)((timestamp >> 32) & 0xFFFF);
     uuid.time_hi_and_version = (unsigned short)(timestamp >> 48) ;
     }
@@ -3109,3 +3242,5 @@ return sPtr;
 
 /* eof */
 
+
+
