@@ -1,6 +1,6 @@
 /* n-list.c
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,20 +20,19 @@
 
 #include "newlisp.h"
 #include "protos.h"
-#include <search.h>
 
 extern SYMBOL * starSymbol;
 extern SYMBOL * plusSymbol;
-extern SYMBOL * sysSymbol[];
+extern CELL * countCell;
 
 extern CELL * firstFreeCell;
-extern SYMBOL * dolistIdxSymbol;
 
 /* following used in count, difference, intersect, unique and sort 8.6.2 */
 CELL * * listToSortedVector(CELL * list, ssize_t * length, CELL * func, int indexFlag);
 CELL * resortVectorToList(CELL * * vector, ssize_t length);
 void binsort(CELL * * x, ssize_t n, CELL * pCell);
 
+
 CELL * p_map(CELL * params)
 {
 CELL * argsPtr;
@@ -51,7 +50,9 @@ funcPtr = evaluateExpression(params);
 
 /* get first of argument lists */
 params = getEvalDefault(params->next, &cell);
-argsPtr = cell = copyCell(cell);
+
+argsPtr = cell = (cell->type == CELL_ARRAY) ?  
+    arrayList(cell, FALSE) : copyCell(cell);
 
 if(!isList(cell->type))
     return(errorProcExt(ERR_LIST_EXPECTED, cell));
@@ -60,9 +61,10 @@ if(!isList(cell->type))
 while (params != nilCell)
     {
     params = getEvalDefault(params, &results);
-    cell->next = copyCell(results);
-    cell = cell->next;
+    cell->next = (results->type == CELL_ARRAY) ? 
+        arrayList(results, FALSE) : copyCell(results);
 
+    cell = cell->next;
     if(!isList(cell->type))
         return(errorProcExt(ERR_LIST_EXPECTED, results));
     }
@@ -102,12 +104,7 @@ while(argsPtr->contents != (UINT)nilCell) /* for all instances of a arg */
         }
 
     cell = evaluateExpression(expr);
-    /* don't use optimized copyCell()
-    if(cell->type == CELL_STRING)
-        cell = stuffStringN((char *)cell->contents, cell->aux - 1);
-    else 
-    */
-        cell = copyCell(cell);
+    cell = copyCell(cell);
     while(resultStackIdx > resultIdxSave)
             deleteList(popResult());
 
@@ -414,7 +411,7 @@ switch(pattern->type)
 
     case CELL_QUOTE:
     case CELL_EXPRESSION:
-    case CELL_MACRO:
+    case CELL_FEXPR:
     case CELL_LAMBDA:
         /* compare subexpressions */
         if(list->type == pattern->type)
@@ -740,103 +737,6 @@ return(result);
 }
 
 
-#ifdef TSEARCH_COUNT
-
-/* not used anymore because of memory leaks,
-   because tree cannot be properly destroyed 
-*/
-void freeTnode(void * node) {};
-
-typedef struct 
-    {
-    UINT type;
-    UINT next;
-    UINT aux;
-    UINT contents;
-    } COUNTCELL;
-
-CELL * p_count(CELL * params)
-{
-CELL * items;
-CELL * list;
-CELL * result;
-CELL * * counts;
-ssize_t lengthItems;
-ssize_t i = 0; 
-int flag = FALSE;
-CELL * cell;
-COUNTCELL * count;
-void * root = NULL;
-void * key;
-
-params = getListHead(params, &items);
-getListHead(params, &list);
-
-result = getCell(CELL_EXPRESSION);
-
-if(items == nilCell)
-    return(result);
-    
-if(items == list)
-    {
-    flag = TRUE;
-    items = copyList(list);
-    }
-    
-lengthItems = listlen(items);
-counts = (CELL * *)callocMemory(lengthItems * sizeof(CELL *));
-
-cell = items;
-for(i = 0; i < lengthItems; i++)
-    {
-    counts[i] = copyCell(cell); 
-    counts[i]->next = NULL;
-    key = tsearch(counts[i], &root, (int (*)(const void *, const void *))compareCells);
-    if(key == NULL)
-        {
-        freeMemory(counts);
-        errorProc(ERR_NOT_ENOUGH_MEMORY);
-        }
-    cell = cell->next;
-    }
-
-cell = list;
-while(cell != nilCell)
-    {
-    key = tfind(cell, &root, (int (*)(const void *, const void *))compareCells);
-    if(key != NULL)
-        { 
-        count = (COUNTCELL *)*(CELL * *)key;
-        count->next++;
-        }
-    cell = cell->next;
-    }
-
-cell = stuffInteger((UINT)counts[0]->next);
-result->contents = (UINT)cell;
-for(i = 1; i < lengthItems; i++)
-    {
-    cell->next = stuffInteger((UINT)counts[i]->next);
-    cell = cell->next;
-    }
-
-for(i = 0; i < lengthItems; i++)
-    {
-    counts[i]->next = nilCell;
-    deleteList(counts[i]);
-    }
-
-freeMemory(counts);
-if(flag) deleteList(items);
-
-tdestroy(root, freeTnode);
-
-return(result);
-}
-
-#endif
-
-
 CELL * p_popAssoc(CELL * params)
 {
 CELL * key;
@@ -867,7 +767,10 @@ while(key != nilCell)
     while(list != nilCell)
         {
         if(isList(list->type))
+            {
+            list->aux = (UINT)nilCell; /* undo last element optimization */
             if(compareCells(key, (CELL *)list->contents) == 0) break;
+            }
         previous = list;
         list = list->next;
         }
@@ -1034,7 +937,7 @@ if(isList(cell->type))
     cell->contents = (UINT)vector[0];
     freeMemory(vector);
     }
-else if(isArray(cell->type))
+else if(cell->type == CELL_ARRAY)
     {
     vector = (CELL **)cell->contents;
     length = (cell->aux - 1) / sizeof(UINT);
@@ -1068,13 +971,14 @@ for(i = 0; i < *length; i++)
     list = list->next;
     if(indexFlag) prev->next = (void *)i;
     }
-
 if(func != nilCell && func != NULL)
     {
     func = evaluateExpression(func);
     if(func->type == CELL_SYMBOL) 
         func = (CELL*)((SYMBOL *)func->contents)->contents;
+    func = copyCell(func);
     binsort(vector, *length, func);
+    deleteList(func);
     } 
 else
     binsort(vector, *length, NULL);
@@ -1122,6 +1026,7 @@ if(isNumber(cell->type))
 else if(isList(cell->type))
     {
     params = (CELL*)cell->contents;
+    if(params == nilCell) return(list);
     params = getIntegerExt(params, (UINT *)&index, FALSE);
     evalFlag = FALSE;
     }
@@ -1169,8 +1074,8 @@ int intFlag;
 
 if((intFlag = (((CELL*)params->next)->next == nilCell)))
     {
-    params = getInteger64(params, &fromInt64);
-    getInteger64(params, &toInt64);
+    params = getInteger64Ext(params, &fromInt64, TRUE);
+    getInteger64Ext(params, &toInt64, TRUE);
     stepCnt = (fromInt64 > toInt64) ? fromInt64 - toInt64 : toInt64 - fromInt64;
     cell = stuffInteger64(fromInt64);
     }
@@ -1187,7 +1092,7 @@ else
     step = (fromFlt > toFlt) ? -step : step;
     cntFlt = (fromFlt < toFlt) ? (toFlt - fromFlt)/step : (fromFlt - toFlt)/step;
     stepCnt = (cntFlt > 0.0) ? floor(cntFlt + 0.0000000001) : floor(-cntFlt + 0.0000000001);
-    cell = stuffFloat(&fromFlt);
+    cell = stuffFloat(fromFlt);
     }
 sequence = makeCell(CELL_EXPRESSION, (UINT)cell);
 
@@ -1203,11 +1108,12 @@ for(i = 1; i <= stepCnt; i++)
     else
         {
         interval = fromFlt + i * step;
-        cell->next = stuffFloat(&interval);
+        cell->next = stuffFloat(interval);
         }
     cell = cell->next;
     }
 
+sequence->aux = (UINT)cell; /* last element optimization */
 return(sequence);
 }
 
@@ -1218,51 +1124,119 @@ return(sequence);
 #define FILTER_FOR_ALL 3
 #define FILTER_EXISTS 4
 
+/* on EMSCRIPTEN, when compiling with -O1 or -O2, this is necessary
+   optimization messes up setjmp/longjmp
+*/
+
+#ifdef EMSCRIPTEN
+CELL * filterIndex(CELL * pCell, CELL * args, int mode);
+#else
 CELL * filterIndex(CELL * params, int mode);
+#endif
+
 
 CELL * p_filter(CELL * params)
 {
+#ifdef EMSCRIPTEN
+CELL * pCell;
+CELL * args;
+pCell = evaluateExpression(params);
+getEvalDefault(params->next, &args);
+if(!isList(args->type))
+    return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+return filterIndex(pCell, args, FILTER_FILTER);
+#else
 return filterIndex(params, FILTER_FILTER);
+#endif
 }
 
 CELL * p_index(CELL * params)
 {
+#ifdef EMSCRIPTEN
+CELL * pCell;
+CELL * args;
+pCell = evaluateExpression(params);
+getEvalDefault(params->next, &args);
+if(!isList(args->type))
+    return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+return filterIndex(pCell, args, FILTER_INDEX);
+#else
 return filterIndex(params, FILTER_INDEX);
+#endif
 }
 
 CELL * p_clean(CELL * params)
 {
+#ifdef EMSCRIPTEN
+CELL * pCell;
+CELL * args;
+pCell = evaluateExpression(params);
+getEvalDefault(params->next, &args);
+if(!isList(args->type))
+    return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+return filterIndex(pCell, args, FILTER_CLEAN);
+#else
 return filterIndex(params, FILTER_CLEAN);
+#endif
 }
 
 CELL * p_exists(CELL * params)
 {
+#ifdef EMSCRIPTEN
+CELL * pCell;
+CELL * args;
+pCell = evaluateExpression(params);
+getEvalDefault(params->next, &args);
+if(!isList(args->type))
+    return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+return filterIndex(pCell, args, FILTER_EXISTS);
+#else
 return filterIndex(params, FILTER_EXISTS);
+#endif
 }
 
 CELL * p_forAll(CELL * params)
 {
-return filterIndex(params, FILTER_FOR_ALL);
-}
-
-CELL * filterIndex(CELL * params, int mode)
-{
-CELL * expr;
+#ifdef EMSCRIPTEN
 CELL * pCell;
 CELL * args;
+pCell = evaluateExpression(params);
+getEvalDefault(params->next, &args);
+if(!isList(args->type))
+    return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+return filterIndex(pCell, args, FILTER_FOR_ALL);
+#else
+return filterIndex(params, FILTER_FOR_ALL);
+#endif
+}
+
+#ifdef EMSCRIPTEN
+CELL * filterIndex(CELL * pCell, CELL * args, int mode)
+#else
+CELL * filterIndex(CELL * params, int mode)
+#endif
+{
+CELL * expr;
+#ifndef EMSCRIPTEN
+CELL * pCell;
+CELL * args;
+#endif
 CELL * resultList = NULL;
 CELL * result;
 CELL * cell;
-ssize_t count;
 UINT * resultIndexSave;
 jmp_buf errorJumpSave;
+ssize_t count;
 int errNo, trueFlag;
 
+#ifndef EMSCRIPTEN
 pCell = evaluateExpression(params);
 getEvalDefault(params->next, &args);
 
 if(!isList(args->type))
     return(errorProcExt(ERR_LIST_EXPECTED, params->next));
+#endif
+
 args = (CELL *)args->contents;
 
 result = NULL;
@@ -1409,6 +1383,7 @@ while(list != nilCell)
             }   
 
         if(mode == REF_SINGLE) return;
+        countCell->contents++;
         }
 
     if(isList(list->type))
@@ -1485,6 +1460,7 @@ return(reference(params, REF_SINGLE));
 
 CELL * p_refAll(CELL * params)
 {
+countCell->contents = 0;
 return(reference(params, REF_ALL));
 }
 
@@ -1504,6 +1480,7 @@ while(list != nilCell)
         *count += 1;
         updateCell(list, new);
         if(mode == SETREF_SINGLE) return(list);
+        countCell->contents++;
         }
     else if(isList(list->type))
         {
@@ -1558,6 +1535,7 @@ return(setRef(params, SETREF_SINGLE));
 
 CELL * p_setRefAll(CELL * params)
 {
+countCell->contents = 0;
 return(setRef(params, SETREF_ALL));
 }
 
@@ -1584,7 +1562,11 @@ if(val != nilCell)
         else
             deleteList((CELL *)cell->contents);
         }
-    else if(cell->type == CELL_STRING || cell->type == CELL_DYN_SYMBOL) 
+    else if(cell->type == CELL_STRING || cell->type == CELL_DYN_SYMBOL 
+#ifdef BIGINT
+            || cell->type == CELL_BIGINT
+#endif
+            ) 
         freeMemory( (void *)cell->contents);
 
     cell->type = new->type;
@@ -1603,12 +1585,12 @@ if(val != nilCell)
 itSymbol->contents = (UINT)nilCell;
 }
 
-void flat(CELL * list, CELL * result, CELL * * next)
+void flat(CELL * list, CELL * result, CELL * * next, UINT recursion)
 {
 while(list != nilCell)
     {
-    if(isList(list->type))
-        flat((CELL*)list->contents, result, next);
+    if(isList(list->type) && recursion != 0)
+        flat((CELL*)list->contents, result, next, recursion - 1);
     else
         {
         if(*next == NULL)
@@ -1633,17 +1615,46 @@ CELL * p_flat(CELL * params)
 CELL * list;
 CELL * result;
 CELL * next = NULL;
+UINT recursion = -1;
 
-getListHead(params, &list);
+params = getListHead(params, &list);
+if(params != nilCell)
+    getInteger(params, &recursion);
 
 result = getCell(CELL_EXPRESSION);
 
-flat(list, result, &next);
+flat(list, result, &next, recursion);
 
 return(result);
 }
 
+/* 
+    (collect <expr> [int-max-count]) 
+    collect results of evaluating <expr> while not nil
+    and optional max count is not reached
+*/
+CELL * p_collect(CELL * params)
+{
+CELL * result;
+CELL * cell;
+UINT * resultIdxSave = resultStackIdx;
+UINT count = MAX_LONG;
 
+result = getCell(CELL_EXPRESSION);
+if(params->next != nilCell)
+    getInteger(params->next, &count);
+
+while(count > 0)
+    {
+    cell = evaluateExpression(params);
+    if(isNil(cell)) break;
+    addList(result, copyCell(cell));
+    --count;
+    cleanupResults(resultIdxSave);
+    }
+
+return(result);
+}
 
 
 /* --------------------------------- array routines ------------------------- */
@@ -1768,10 +1779,10 @@ getEvalDefault(params, &array);
 if(array->type != CELL_ARRAY)
     return(errorProcExt(ERR_ARRAY_EXPECTED, params));
 
-return(arrayList(array));
+return(arrayList(array, TRUE));
 }
 
-CELL * arrayList(CELL * array)
+CELL * arrayList(CELL * array, int flag)
 {
 CELL * list = NULL;
 CELL * * addr;
@@ -1785,8 +1796,8 @@ size = (array->aux - 1) / sizeof(UINT);
 while(size--)
     {
     cell = *(addr++);
-    if(cell->type == CELL_ARRAY)
-        new = arrayList(cell);
+    if((cell->type == CELL_ARRAY) && flag)
+        new = arrayList(cell, flag);
     else
         new = copyCell(cell);
     if(list == NULL)
@@ -1982,7 +1993,7 @@ void printArray(CELL * array, UINT device)
 {
 CELL * list;
 
-list = arrayList(array);
+list = arrayList(array, TRUE);
 
 printExpression(list, device);
 
@@ -2010,6 +2021,7 @@ ssize_t size, index;
 int evalFlag;
 
 list = evaluateExpression(params);
+
 if(isNumber(list->type))
     {
     getIntegerExt(list, (UINT *)&index, FALSE);
@@ -2019,6 +2031,7 @@ if(isNumber(list->type))
 else if(isList(list->type))
     {
     params = (CELL*)list->contents;
+    if(params == nilCell) return(cell);
     params = getIntegerExt(params, (UINT *)&index, FALSE);
     evalFlag = FALSE;
     }
