@@ -1,7 +1,7 @@
 /* nl-liststr.c --- newLISP primitives handling lists and strings
 
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@
 #include "protos.h"
 
 extern CELL * lastCellCopied;
+extern CELL * countCell;
 extern SYMBOL * sysSymbol[];
+extern SYMBOL * countSymbol;
 extern void printResultStack();
 
 /* used only on string indices */
@@ -53,13 +55,13 @@ return(count);
 }
 
 #ifdef LINUX
-extern char * strcasestr(char * haystack, char * needle);
+extern char * strcasestr(const char * haystack, const char * needle);
 #endif
 CELL * p_member(CELL * params)
 {
 CELL * key;
 CELL * list;
-long options  = -1;
+INT options  = -1;
 char * ptr;
 ssize_t pos;
 
@@ -105,19 +107,55 @@ CELL * p_length(CELL * params)
 {
 size_t length;
 SYMBOL * symbol;
+#ifdef BIGINT
+int * bigintPtr;
+int * result;
+int len;
+#else
+INT64 number;
+double fnum;
+#endif
 
 params = evaluateExpression(params);
 length = 0;
 switch(params->type)
     {
+#ifdef BIGINT
     case CELL_LONG:
-        length = sizeof(UINT); break;
 #ifndef NEWLISP64
     case CELL_INT64:
-        length = sizeof(INT64); break;
 #endif
     case CELL_FLOAT:
-        length = sizeof(double); break;
+    case CELL_BIGINT:   
+        result = getBigintSizeDirect(params, &bigintPtr, &len);
+        length = lengthBigint(bigintPtr, len);
+        if(result) free(result);
+        break; 
+#else /* not BIGINT */
+    case CELL_LONG:
+#ifndef NEWLISP64
+    case CELL_INT64:
+#endif
+        getInteger64Ext(params, &number, FALSE);
+        if(number == 0)
+            length = 1;
+        else
+            {
+            if(number < 0) number = - number;
+            length = log(number) / log(10) + 1.5;
+            }
+        break;
+    case CELL_FLOAT:
+        getFloat(params, &fnum);
+        if(fnum == 0.0)
+            length = 1;
+        else
+            {
+            if(fnum < 0.0) fnum = - fnum;
+            length = log(fnum) / log(10) + 1.5;
+            }
+        break;
+#endif /* not BIGINT */
     case CELL_STRING:
         length = params->aux - 1; break;
     case CELL_CONTEXT:
@@ -128,6 +166,8 @@ switch(params->type)
             length = params->aux - 1;
         else if(isList(params->type))
             length = listlen((CELL *)params->contents);
+        else if(params->type == CELL_ARRAY)
+            length = (params->aux -1) / sizeof(UINT);
         break;
     case CELL_SYMBOL:
         symbol = (SYMBOL *)params->contents;
@@ -138,7 +178,7 @@ switch(params->type)
         break;
     case CELL_EXPRESSION:
     case CELL_LAMBDA:
-    case CELL_MACRO:
+    case CELL_FEXPR:
             length = listlen((CELL *)params->contents);
         break;
     case CELL_ARRAY:
@@ -146,6 +186,7 @@ switch(params->type)
     default:
         break;
     }
+
 return(stuffInteger(length));
 }
 
@@ -191,6 +232,7 @@ if(list == NULL)
     return(getCell(CELL_EXPRESSION));
 
 symbolCheck = NULL;
+list->aux = (UINT)lastCellCopied; /* last element optimization */
 return(list);
 }
 
@@ -198,7 +240,7 @@ return(list);
 CELL * appendString(CELL * cell, CELL * list, char * joint, size_t jointLen, int trailJoint, int evalFlag)
 { 
 CELL * result;
-STREAM stream = {0, NULL, NULL, 0, 0};
+STREAM stream = {NULL, NULL, 0, 0, 0};
 char * sPtr;
 size_t len;
 
@@ -247,11 +289,13 @@ if((symbolRef = symbolCheck))
         return(errorProcExt2(ERR_SYMBOL_PROTECTED, stuffSymbol(symbolRef)));
     if(isNil((CELL *)symbolRef->contents))
         {
-        deleteList((CELL*)symbolRef->contents);
-        head = evaluateExpression(params);
+        head = evaluateExpression(params); /* extension */
         if(isList(head->type) || head->type == CELL_STRING)
+            {
             target = copyCell(head);
-        symbolRef->contents = (UINT)target;
+            deleteList((CELL*)symbolRef->contents);
+            symbolRef->contents = (UINT)target;
+            }
         params = params->next;
         }
     }
@@ -363,11 +407,9 @@ else if(list->type == CELL_ARRAY)
 else if(list->type == CELL_STRING)
     {
     list = implicitIndexString(list, params);
-    if((symbolCheck = symbolRef))
-        {
-        pushResult(list);
-        pushResultFlag = FALSE;
-        }
+    symbolCheck = symbolRef;
+    pushResult(list);
+    pushResultFlag = FALSE;
     return(list);
     }
         
@@ -462,10 +504,10 @@ if(index == -1)
         }
     }
     
-list->aux = (UINT)nilCell; /* undo last element optimization */
 
 while(isList(list->type))
     {
+    list->aux = (UINT)nilCell; /* undo last element optimization */
     cell = list;
     list = (CELL *)list->contents;
 
@@ -558,7 +600,6 @@ if(params == nilCell)
     }
 else
     {
-    list->aux = (UINT)nilCell; /* undo last element optimization */
     cell = (CELL*)params->next;
     params = evaluateExpression(params);
     if(isList(params->type))
@@ -576,6 +617,7 @@ else
 
 while(isList(list->type))
     {
+    list->aux = (UINT)nilCell; /* undo last element optimization */
     cell = list;
     list = (CELL *)list->contents;
 
@@ -589,7 +631,7 @@ while(isList(list->type))
     if(list == nilCell) 
         errorProc(ERR_LIST_INDEX_INVALID);
 
-    if(params == nilCell || !isList(list->type))  break;
+    if(params == nilCell || !isList(list->type)) break;
     params = getIntegerExt(params, (UINT*)&index, evalFlag);
     }
 
@@ -892,9 +934,12 @@ CELL * list;
 CELL * previous;
 CELL * next;
 char * str;
-size_t len, tmp;
 char * left;
 char * right;
+CELL * * addr;
+CELL * * leftA;
+CELL * * rightA;
+size_t len, tmp;
 
 cell = params;
 getEvalDefault(params, &list);
@@ -933,7 +978,24 @@ else if(list->type == CELL_STRING)
         right--;
         }
     }
-else return(errorProcExt(ERR_LIST_OR_STRING_EXPECTED, cell));
+
+else if(list->type == CELL_ARRAY)
+    {
+    addr = (CELL * *)list->contents;
+    len = (list->aux - 1) / sizeof(UINT);
+    leftA = addr;
+    rightA = leftA + len - 1;
+    while(leftA < rightA)
+        {
+        cell = *leftA;
+        *leftA = *rightA;
+        *rightA = cell;
+        leftA++;
+        rightA--;
+        }
+    }
+    
+else return(errorProcExt(ERR_ARRAY_LIST_OR_STRING_EXPECTED, cell));
 
 pushResultFlag = FALSE;
 return(list);
@@ -973,7 +1035,7 @@ CELL * next;
 CELL * keyCell;
 CELL * funcCell;
 size_t size;
-long options = -1;
+INT options = -1;
 size_t offset = 0;
 UINT * resultIdxSave;
 
@@ -993,7 +1055,8 @@ if(keyCell->type == CELL_STRING && next->type == CELL_STRING)
         if(offset > size) offset = size;
         params = evaluateExpression(params);
         if(!isNil(params))
-            getIntegerExt(params, (UINT *)&options, FALSE);
+            /* 10.6.1 also accept string for options */
+            parseRegexOptions(params, (UINT *)&options, FALSE);
         }
 
     if(options == -1)
@@ -1005,8 +1068,8 @@ if(keyCell->type == CELL_STRING && next->type == CELL_STRING)
 else
     {
     /* list mode with optional functor */
-
-    if(!isList(next->type)) return(nilCell);
+    if(!isList(next->type)) 
+        return(errorProcExt(ERR_LIST_OR_STRING_EXPECTED, next));
     next = (CELL *)next->contents;
     found = 0;
 
@@ -1015,9 +1078,12 @@ else
     else funcCell = NULL;
 
     /* do regex when first arg is string and option# is present */
-    if(funcCell && isNumber(funcCell->type) && keyCell->type == CELL_STRING)
+    if(funcCell && 
+        (isNumber(funcCell->type) || funcCell->type == CELL_STRING) &&
+        keyCell->type == CELL_STRING)
         {
-        getIntegerExt(funcCell, (UINT*)&options, FALSE);
+        /* 10.6.1 also accept string for options */
+        parseRegexOptions(funcCell, (UINT *)&options, FALSE);
         key = (char *)keyCell->contents;
         while(next != nilCell)
             {
@@ -1059,7 +1125,7 @@ return(stuffInteger(found + offset));
 
 CELL * findAllString(char * pattern, char * str, size_t size, CELL * params)
 {
-long options = 0;
+INT options = 0;
 ssize_t findPos = -1;
 ssize_t lastPos = -1;
 CELL * result = nilCell;
@@ -1074,9 +1140,11 @@ int offset = 0;
 
 exprCell = params;
 if((params = params->next) != nilCell)
-    getInteger(params, (UINT *)&options);
+    /* 10.6.1 also accept string for options */
+    parseRegexOptions(params, (UINT *)&options, TRUE);
 
 resultIdxSave = resultStackIdx;
+countCell->contents = 0;
     
 if(exprCell != nilCell)
     {
@@ -1092,6 +1160,7 @@ if(exprCell != nilCell)
 while( (findPos = searchBufferRegex(str, offset, pattern, (int)size, options, &len)) 
        != -1)
     {
+    countCell->contents++;
     if(exprCell != nilCell)
         {
         itSymbol->contents = sysSymbol[0]->contents;
@@ -1123,6 +1192,7 @@ while( (findPos = searchBufferRegex(str, offset, pattern, (int)size, options, &l
 
     FINDALL_CONTINUE:
     offset = (findPos + len);
+    if(findPos > size) break;
     cleanupResults(resultIdxSave);
     }
 
@@ -1131,7 +1201,7 @@ itSymbol->contents = (UINT)nilCell;
 if(exprCell != nilCell)
     memcpy(errorJump, errorJumpSave, sizeof(jmp_buf));
 
-return(result);
+return(result == nilCell ? getCell(CELL_EXPRESSION) : result);
 }
 
 
@@ -1147,10 +1217,8 @@ int errNo;
 
 funcCell = evaluateExpression(exprCell->next);
 resultIdxSave = resultStackIdx;
+countCell->contents = 0;
 
-if(funcCell == nilCell && !isList(pattern->type))
-    return(errorProcExt(ERR_LIST_EXPECTED, pattern));
-    
 result = getCell(CELL_EXPRESSION);
 
 memcpy(errorJumpSave, errorJump, sizeof(jmp_buf));
@@ -1165,16 +1233,25 @@ while(list != nilCell)
     {
     if(funcCell == nilCell)
         {
-        /* match only takes lists*/
-        if(!isList(list->type))
-            goto CONTINUE_NEXT;
+        /* added in 10.4.7, this makes exp in func in 3rd syntax optional */
+        if(!isList(pattern->type))
+            {
+            if(compareFunc(pattern, list, NULL) != 0)
+                goto CONTINUE_NEXT;
+            }
+        else 
+            {
+            /* match only takes lists*/
+            if(!isList(list->type))
+                goto CONTINUE_NEXT;
 
-        match = patternMatchL((CELL *)pattern->contents, (CELL *)list->contents, TRUE);
+            match = patternMatchL((CELL *)pattern->contents, (CELL *)list->contents, TRUE);
 
-        if(match == NULL || match == nilCell)
-            goto CONTINUE_NEXT;
+            if(match == NULL || match == nilCell)
+                goto CONTINUE_NEXT;
 
-        deleteList(match);
+            deleteList(match);
+            }
         }
     else
         {
@@ -1183,9 +1260,7 @@ while(list != nilCell)
             goto CONTINUE_NEXT;
         }
 
-    /* take out sysSymbol in future, should only be used in regex */
-    deleteList((CELL*)sysSymbol[0]->contents);
-    sysSymbol[0]->contents = (UINT)copyCell(list);
+    countCell->contents++;
     itSymbol->contents = (UINT)list;
 
     if(exprCell != nilCell)
@@ -1194,6 +1269,7 @@ while(list != nilCell)
         exprRes = list;
 
     addList(result, copyCell(exprRes));
+    /* increment $count here */
 
     CONTINUE_NEXT:  
     list = list->next;
@@ -1328,7 +1404,7 @@ CELL * startsEndsWith(CELL * params, int type)
 char * string;
 char * key;
 char * keydollar;
-long options = -1;
+INT options = -1;
 size_t slen, pos;
 int klen;
 CELL * cell, * list;
@@ -1359,7 +1435,8 @@ else
 if(cell->next != nilCell)
     {
     cell = evaluateExpression(cell->next);
-    getIntegerExt(cell, (UINT*)&options, FALSE);
+    /* 10.6.1 also accept string for options */
+    parseRegexOptions(cell, (UINT *)&options, FALSE);
     }
 
 klen = strlen(key);
@@ -1425,9 +1502,8 @@ CELL * newList;
 char * keyStr;
 char * buff;
 char * newBuff;
-UINT cnt; 
 size_t newLen;
-long options;
+INT options;
 UINT * resultIdxSave;
 SYMBOL * refSymbol;
 
@@ -1439,7 +1515,7 @@ refSymbol = symbolCheck;
 if(symbolCheck && (isProtected(symbolCheck->flags) || isBuiltin(symbolCheck->flags)))
     return(errorProcExt2(ERR_SYMBOL_PROTECTED, stuffSymbol(symbolCheck)));
 
-cnt = 0;
+countCell->contents = 0;
 resultIdxSave = resultStackIdx;
 if(isList(cell->type))
     {
@@ -1459,14 +1535,10 @@ if(isList(cell->type))
 COMPARE_START:
     if(compareFunc(keyCell, list, funcCell) == 0)
         {
+        countCell->contents++;
         if(repCell != NULL)
             {
-            /* perhaps take out usage of sysSymbol[0] 
-               should only be used for regex replacements 
-               then $it doesn't need to be a copy */
-            deleteList((CELL*)sysSymbol[0]->contents);
-            itSymbol->contents = (UINT)copyCell(list);
-            sysSymbol[0]->contents = itSymbol->contents;
+            itSymbol->contents = (UINT)list;
             cell->contents = (UINT)copyCell(evaluateExpression(repCell));
             cell = (CELL*)cell->contents;
             cell->next = list->next;
@@ -1474,9 +1546,8 @@ COMPARE_START:
         else /* remove mode */
             cell->contents = (UINT)list->next;
 
-        list->next = nilCell; /* decouple */
+        list->next = nilCell; /* decouple  and delete old */
         deleteList(list);
-        cnt++;
 
         if(repCell != NULL)
             list = cell;
@@ -1492,21 +1563,17 @@ COMPARE_START:
         {
         if(compareFunc(keyCell, list->next, funcCell) == 0)
             {
+            countCell->contents++;
             cell = list->next;  /* cell = old elmnt */
             if(repCell != NULL)
                 {
-                /* perhaps take out usage of sysSymbo[0]
-                should only be used for regex replacements */
-                deleteList((CELL*)sysSymbol[0]->contents);
-                itSymbol->contents = (UINT)copyCell(cell);
-                sysSymbol[0]->contents = itSymbol->contents;
+                itSymbol->contents = (UINT)cell;
                 list->next = copyCell(evaluateExpression(repCell));
                 list = list->next;
                 }
             list->next = cell->next;
             cell->next = nilCell;
             deleteList(cell);
-            cnt++;
             }       
         else    
             list = list->next;
@@ -1514,9 +1581,6 @@ COMPARE_START:
         cleanupResults(resultIdxSave);
         }
 
-    deleteList((CELL*)sysSymbol[0]->contents);  
-    /* sysSymbol[0] should not be used here, introduce $count ? */
-    sysSymbol[0]->contents = (UINT)stuffInteger(cnt);
     itSymbol->contents = (UINT)nilCell;
     symbolCheck = refSymbol;
     pushResultFlag = FALSE;
@@ -1536,10 +1600,11 @@ if(cell->type == CELL_STRING)
             
     options = -1;
     if(repCell->next != nilCell)
-            getInteger(repCell->next, (UINT*)&options);
+            /* 10.6.1 also accept string for options */
+            parseRegexOptions(repCell->next, (UINT *)&options, TRUE);
 
     newBuff = replaceString(keyStr, keyCell->aux - 1, 
-                           buff, (size_t)cell->aux -1, repCell, &cnt, options, &newLen);
+                           buff, (size_t)cell->aux -1, repCell, &countCell->contents, options, &newLen);
     if(newBuff != NULL)
         {
         freeMemory(buff);
@@ -1547,8 +1612,6 @@ if(cell->type == CELL_STRING)
         cell->aux = newLen + 1;
         }
 
-    deleteList((CELL*)sysSymbol[0]->contents);  
-    sysSymbol[0]->contents = (UINT)stuffInteger(cnt);
     symbolCheck = refSymbol;
     pushResultFlag = FALSE;
     return(cell);
@@ -1565,6 +1628,7 @@ CELL * cell;
 CELL * list;
 CELL * previous;
 CELL * last = NULL;
+char * str;
 size_t length, index;
 size_t count;
 
@@ -1580,16 +1644,14 @@ if(symbolCheck && isProtected(symbolCheck->flags))
 if(list->type == CELL_STRING)
     {   
     length = list->aux - 1;
-    if((count = adjustCount(count, length)) == 0) 
+    if((count = adjustCount(count, length)) != 0) 
         {
-        pushResultFlag = FALSE;
-        return(list);
+    	str = allocMemory(list->aux);  
+    	memcpy(str, (char *)(list->contents + length - count), count);
+    	memcpy(str + count, (char *)list->contents, length - count);
+    	memcpy((char*)list->contents, str, length);
+    	free(str);
         }
-    cell = copyCell(list);  
-    memcpy((char*)cell->contents, (char *)(list->contents + length - count), count);
-    memcpy((char*)(cell->contents + count), (char *)list->contents, length - count);
-    memcpy((char*)list->contents, (char*)cell->contents, length);
-    deleteList(cell);
     pushResultFlag = FALSE;
     return(list);
     }   
@@ -1628,6 +1690,7 @@ last->next = (CELL *)list->contents;
 list->contents = (UINT)cell;
 
 pushResultFlag = FALSE;
+list->aux = (UINT)previous; /* last element optimization */
 return(list);
 }
 
