@@ -1,6 +1,6 @@
 /* nl-matrix.c --- matrix functions for newLISP
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "newlisp.h"
 #include "protos.h"
+#include "float.h"
 
 double * * multiply(double ** A, double ** B, int m, int n, int k, int l);
 double * * invert(double * * A, int n, int * err, double tiny);
@@ -38,6 +39,9 @@ void freeMatrix(double * * m, int rows);
 
 /* since version 7.4.8 'transpose' tramspose any matrix not only
    matrices containing numbers.
+
+   some of the algorithms used come from C LAPACK and index
+   starting 1 not 0 as usual in C
 */ 
 CELL * p_matTranspose(CELL * params)
 {
@@ -230,13 +234,13 @@ for(i = 1; i <= n; i++)
 free(indx);
 freeMatrix(M, n);
 
-return(stuffFloat(&d));
+return(stuffFloat(d));
 }
 
 
 CELL * p_matScalar(CELL * params)
 {
-CELL * param;
+CELL * op;
 double * * A = NULL;
 double * * B = NULL;
 double * * M = NULL;
@@ -245,13 +249,13 @@ int type;
 int n, m, k, l, err = 0;
 
 CELL * result;
-param = params;
-param = evaluateExpression(param);
+op = params;
+op = evaluateExpression(op);
 
-if(param->type == CELL_SYMBOL)
-    type = *((SYMBOL *)param->contents)->name;
-else if(param->type == CELL_PRIMITIVE)
-    type = *(char *)param->aux;
+if(op->type == CELL_SYMBOL)
+    type = *((SYMBOL *)op->contents)->name;
+else if(op->type == CELL_PRIMITIVE)
+    type = *(char *)op->aux;
 else
     return(errorProcExt(ERR_ILLEGAL_TYPE, params));
 
@@ -303,17 +307,8 @@ else
             case '-': M[k][l] = A[k][l] - B[k][l]; break;
             case '*': M[k][l] = A[k][l] * B[k][l]; break;
             case '/': M[k][l] = A[k][l] / B[k][l]; break;
-/* before 10.2.0
-                { 
-                if(B[k][l] == 0)
-                    return(errorProc(ERR_MATH));
-                else
-                    M[k][l] = A[k][l] / B[k][l];
-                }
-                break;
-*/
             default:
-            return(errorProcExt(ERR_ILLEGAL_TYPE, params));
+            return(errorProcExt(ERR_ILLEGAL_TYPE, op));
             }
         }
 }
@@ -338,15 +333,15 @@ double * * C;
 int i, j, s;
 double sum;
 
-if((C = allocateMatrix(n, l)) == NULL)
-    return(NULL);
-
 if(m != k)
     {
     errorProc(ERR_WRONG_DIMENSIONS);
     return(NULL);
     }
 
+if((C = allocateMatrix(n, l)) == NULL)
+    return(NULL);
+
 for(i = 1; i <= n; i++)
     {
     for(j = 1; j <= l; j++)
@@ -525,6 +520,266 @@ for (i = n; i >= 1; i--)
     }
 }
 
+/* ------------ KMEANS clustering ------------------------------- */
+
+#ifdef KMEANS
+#define DOUBLE_MAX 1.0e308 /* approximate for IEEE 754 */
+
+/* (kmeans-train data k context [centroidsi [criterium]])
+   data = n * m matrix (list or array)
+   k = number of clusters
+   centroids = k * m matrix of start centroids (optional) (list or array)
+   when centroids then optional convergence criterium, 1e-10 by default
+   the m of optional centroids may be smaller than m in data to force
+   reduction of dimension, e.g. when the last column of date is pre-labeled
+
+   data and optional centroids as either array or list
+
+   if no centroids are given, they are calculated from
+   inital random membership
+
+   returns a list of idecreasing inner SSQs from start to 
+   end of iterations
+
+   result context ctx
+   ctx:centroids (m * k list)
+   ctx:clusters list with k sublists of indices into data (k sublists)
+   cts:labels list with n cluster numbers for each data records  (n list)
+
+   labels go from 1 to k
+
+   NOTE, indices in C-source are not 0-based as usually in C but 
+   go from 1 to N
+*/
+CELL * p_kmeansTrain(CELL * params)
+{
+double * * X; /* data */
+double * * C; /* centroids `*/
+int * labels; /* membership */
+int * counts; /* cluster sizes */
+double * ssqs; /* inner SSQs of clusters */
+double ssqTotal; /* total squared sum inner deviations */
+double ssqRecord; /* inner ssq of a record in a cluster */
+double ssqMin;     /* minimum ssq for one record */
+double ssqPrev = DOUBLE_MAX;
+double dist;
+double crit = 1.0e-10;
+CELL * centroids;
+CELL * clusters;
+CELL * deviations;
+CELL * SSQlist = NULL;
+CELL * membership;
+CELL * * cellArray;
+SYMBOL * ctx;
+SYMBOL * sPtr;
+int n, m, k; /* rows, columns, groups */
+int i, j, l; /* loop imdices */
+int err = 0, type;
+int kc, mc;
+UINT p;
+
+if((X = getMatrix(params, &type, &n, &m, &err)) == NULL)
+    return(errorProcExt(err, params));
+
+params = getInteger(params->next, &p);
+k = p;
+
+if((ctx = getCreateContext(params, TRUE)) == NULL)
+  return(errorProcExt(ERR_SYMBOL_OR_CONTEXT_EXPECTED, params));
+
+membership = getCell(CELL_EXPRESSION);
+sPtr = translateCreateSymbol("labels", CELL_EXPRESSION, ctx, TRUE); 
+assignSymbol(sPtr, membership);
+
+/* make centroid matrix */
+params = params->next;
+if(params == nilCell)
+    C = allocateMatrix(k, m);
+/* or get predefined centroids from user */
+else 
+    {
+    if((C = getMatrix(params, &type, &kc, &mc, &err)) == NULL)  
+        return(errorProcExt(err, params));
+    /* allow forcing less columns via optional centroids mc <= m */
+    if((kc != k) || (mc >  m)) 
+        return(errorProcExt(ERR_WRONG_DIMENSIONS, params));
+    m = mc;
+    /* optionally custom criterium */
+    if(params->next != nilCell)
+        getFloat(params->next, &crit);
+    }
+
+counts = allocMemory((k + 1) * sizeof(int));
+labels = allocMemory((n + 1) * sizeof(int));
+ssqs = callocMemory((k + 1) * sizeof(double));
+
+if(params != nilCell)
+    goto ASSIGN_MEMBERSHIP;
+
+/* asssign random membership */
+for(i = 1; i <= n; i++)
+    labels[i] = (i % k) + 1;
+
+ITERATE_KMEANS:
+/* calc KMEANS centroids from membership */
+for(l = 1; l <= k; l++)     
+    {
+    counts[l] = 0;
+    ssqs[l] = 0.0;
+    for(j = 1; j <= m; j++)
+        C[l][j] = 0;
+    }
+
+for(i = 1; i <= n; i++)
+    {
+    l = labels[i];
+    counts[l] += 1;
+    for(j = 1; j <= m; j++)
+        C[l][j] += X[i][j];
+    }
+
+for(l = 1; l <= k; l++) 
+    for(j = 1; j <= m; j++)
+        C[l][j] = C[l][j] / counts[l];
+
+/* assign each record in X to closest centroid */
+ASSIGN_MEMBERSHIP:
+ssqTotal = 0.0;
+for(i = 1; i <= n; i++)
+    {
+    ssqMin = DOUBLE_MAX;
+    for(l = 1; l <= k; l++)
+        {
+        ssqRecord = 0.0;
+        for(j = 1; j <= m; j++)
+            {
+            dist = X[i][j] - C[l][j];
+            ssqRecord += dist * dist;
+            }
+        if(ssqRecord < ssqMin)
+            {
+            ssqMin = ssqRecord;
+            labels[i] = l;
+            }
+        }
+    ssqTotal += ssqMin;
+    ssqs[labels[i]] += ssqMin;
+    }
+
+if(fabs(ssqPrev - ssqTotal) > crit)
+    {
+    ssqPrev = ssqTotal;
+    if(SSQlist == NULL) SSQlist = getCell(CELL_EXPRESSION);
+    addList(SSQlist, stuffFloat(ssqTotal));
+    goto ITERATE_KMEANS;
+    }
+            
+/* entroid matrix as nested list */
+centroids = matrix2data(C, CELL_EXPRESSION, k, m);
+sPtr = translateCreateSymbol("centroids", CELL_EXPRESSION, ctx, TRUE); 
+assignSymbol(sPtr, centroids);
+/* average intra cluster deviation */
+deviations = getCell(CELL_EXPRESSION);
+sPtr = translateCreateSymbol("deviations", CELL_EXPRESSION, ctx, TRUE); 
+assignSymbol(sPtr, deviations);
+
+/* cluster memberships, for each cluster a list of X data record indices */
+clusters = getCell(CELL_EXPRESSION);
+cellArray = (CELL * *)allocMemory(k * sizeof(CELL));
+for(l = 1; l <= k; l++)
+    {
+    cellArray[l] = getCell(CELL_EXPRESSION);
+    addList(clusters, cellArray[l]);
+    dist = sqrt(ssqs[l] / counts[l]);
+    addList(deviations, stuffFloat(dist)); 
+    }
+for(i = 1; i <= n; i++)
+    {
+    l = labels[i];
+    addList(cellArray[l], stuffInteger(i - 1));
+    }
+
+sPtr = translateCreateSymbol("clusters", CELL_EXPRESSION, ctx, TRUE); 
+assignSymbol(sPtr, clusters);
+
+for(i = 1; i <= n; i++) 
+    addList(membership, stuffInteger(labels[i]));
+
+freeMatrix(C, k);
+freeMatrix(X, n);
+free(cellArray);
+free(ssqs);
+free(labels);
+free(counts);
+
+return(SSQlist);
+}
+
+/* (kmeans-query data-record-vector centroids)
+
+   calculates eucledian distances to all centroids
+   from a previous (kmeans-train data k context [centroids])
+
+   can also be used on the original data to get
+   eucledian distance from one data record to all
+   other records (a KNN fucntion could be built this way)
+
+   (kmeans-query data-record-vector data)
+
+*/
+
+CELL * p_kmeansQuery(CELL * params)
+{
+CELL * data;
+CELL * centroids;
+CELL * dat;
+CELL * result;
+CELL * row;
+CELL * cent;
+double dist;
+double ssq;
+
+params = getEvalDefault(params, &data);
+if(data->type != CELL_EXPRESSION)
+    errorProcExt(ERR_LIST_EXPECTED, data);
+
+getEvalDefault(params, &centroids);
+if(centroids->type == CELL_ARRAY)
+    {
+    centroids = arrayList(centroids, TRUE); 
+    pushResult(centroids);
+    }
+   
+else if(centroids->type != CELL_EXPRESSION)
+    return(errorProcExt(ERR_NOT_MATRIX, centroids));
+
+row = (CELL *)centroids->contents;
+if(row->type != CELL_EXPRESSION)
+    return(errorProcExt(ERR_LIST_EXPECTED, row));
+
+result = getCell(CELL_EXPRESSION);
+while(row != nilCell) /* for each centroid */
+    {
+    cent = (CELL *)row->contents;
+    dat = (CELL *)data->contents;
+    ssq = 0.0;
+    while(cent != nilCell && dat != nilCell) 
+        {
+        dist = fabs(getDirectFloat(cent) - getDirectFloat(dat));
+        ssq += dist * dist; 
+        cent = cent->next;
+        dat = dat->next;
+        }
+    dist = sqrt(ssq);
+    addList(result, stuffFloat(dist));
+    row = row->next;
+    }
+
+return(result);
+}
+
+#endif
+
 /* ------------ make a matrix from list or array expr ----------- */
 
 double * * getMatrix(CELL * params, int * type, int * n, int * m, int *err)
@@ -585,7 +840,7 @@ for(i = 1; i <= n; i++)
 return(matrix);
 }
 
-/* --------------------------- get dimensons or a matrix --------------- */
+/* --------------------------- get dimensons on a matrix --------------- */
 
 int getDimensions(CELL * mat, int * n, int * m)
 {
@@ -764,7 +1019,7 @@ else /* type is CELL_ARRAY */
         for(j = 1; j <= m; j++)
             {
             fnum = matrix[i][j];
-            *(rowAddr + j - 1) = stuffFloat(&fnum);
+            *(rowAddr + j - 1) = stuffFloat(fnum);
             }
         }
     }
