@@ -1,6 +1,6 @@
 /* nl-sock.c
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include "newlisp.h"
 #include <string.h>
 
-#ifdef WIN_32
+#ifdef WINDOWS
 
 #include <winsock2.h>
 #include <ws2tcpip.h>
@@ -55,6 +55,11 @@
 #define IPPROTO_DIVERT 254
 #endif
 
+/* Android needs it */
+#ifndef ICMP6_FILTER
+#define ICMP6_FILTER 1
+#endif
+
 #endif /* end UNIX */
 
 #ifdef CYGWIN
@@ -70,7 +75,7 @@ struct icmp
    unsigned short icmp_id;
    unsigned short icmp_seq;
 };
-#endif
+#endif /* end CYGWIN */
 
 #if defined(SOLARIS) || defined(TRU64) || defined(AIX)
 #include <stropts.h>
@@ -106,8 +111,8 @@ struct icmp
 #define MAX_PENDING_CONNECTS 128
 #define NO_FLAGS_SET 0
 
-#ifdef WIN_32
-#define close closesocket
+#ifdef WINDOWS
+#define close closesocket /* for file operations on Windows use _close */
 #else
 #define SOCKET_ERROR -1
 #define INVALID_SOCKET -1
@@ -120,6 +125,7 @@ IO_SESSION * ioSessions = NULL;
 int isSessionSocket(int sock);
 int getSocketFamily(int sock);
 
+#ifndef EMSCRIPTEN
 #define READY_READ 0
 #define READY_WRITE 1
 
@@ -172,6 +178,7 @@ else
     }
 
 }
+#endif /* ifndef EMSCRIPTEN */
 
 /********************** IO session functions *******************/
 
@@ -248,7 +255,6 @@ while(session)
 return(FALSE);
 }
 
-
 int getSocketFamily(int sock)
 {
 IO_SESSION * session;
@@ -283,8 +289,8 @@ return(NULL);
 }
 
 /* ========================= IO session functions end ===================== */
-
-#ifdef WIN_32
+#ifndef EMSCRIPTEN
+#ifdef WINDOWS
 int ipstrFromSockAddr(struct sockaddr * addr, char * host, int len)
 {
 WSAAddressToString(addr, defaultInLen, NULL, host, (LPDWORD)&len);
@@ -304,8 +310,8 @@ return(TRUE);
 }
 #endif
 
-
-#ifdef WIN_32
+/* ANDROID and WINDOWS need this */
+#ifndef in_port_t
 #define in_port_t short int
 #endif
 
@@ -399,19 +405,18 @@ if(isNumber(cell->type))
 return(errorProcExt(ERR_NUMBER_OR_STRING_EXPECTED, cell));
 } 
 
-
 CELL * p_netConnect(CELL * params)
 {
 CELL * cell;
 char * remoteHostName; 
 UINT portNo;
-UINT topt = CONNECT_TIMEOUT; /* default ms timeout */
-char * protocol = NULL;
+UINT topt = CONNECT_TIMEOUT; /* default ms timeout from newlisp.h */
 int sock;
 int type; 
+int protocol = 0;
 
 params = getString(params, &remoteHostName); 
-#ifndef WIN_32
+#ifndef WINDOWS
 if(params == nilCell)
     {
     if((sock = netConnectLocal(remoteHostName)) == SOCKET_ERROR)
@@ -429,10 +434,11 @@ if(params != nilCell)
     cell = evaluateExpression(params);
     if(cell->type == CELL_STRING)
         {
-        protocol = (char *)cell->contents;
-        *protocol = toupper(*protocol);
+        protocol = toupper(*(char *)cell->contents);
+        if(!(protocol == 'M' || protocol == 'B' || protocol == 'U'))
+            return(errorProc(ERR_INVALID_OPTION));
         type = SOCK_DGRAM;
-        if(*protocol == 'M') /* get ttl */
+        if(protocol == 'M') /* get ttl */
             {
             if(params->next != nilCell)
                 /* topt is ttl in this case */
@@ -458,7 +464,7 @@ return(stuffInteger((UINT)sock));
 }
 
 
-#ifndef WIN_32
+#ifndef WINDOWS
 /* create local domain UNIX socket */
 int netConnectLocal(char * path)
 {
@@ -491,8 +497,8 @@ return(sock);
 
 int blockSocket(int sock)
 {
-#ifdef WIN_32
-unsigned long arg = 0;
+#ifdef WINDOWS
+u_long arg = 0;
 return(ioctlsocket(sock, FIONBIO, &arg));
 #else /* UNIX */
 int arg;
@@ -505,8 +511,8 @@ return(fcntl(sock, F_SETFL, arg));
 #ifdef UNBLOCK
 int unblockSocket(int sock)
 {
-#ifdef WIN_32
-unsigned long arg = 1;
+#ifdef WINDOWS
+u_long arg = 1;
 return(ioctlsocket(sock, FIONBIO, &arg));
 #else /* Unix */
 int arg;
@@ -520,13 +526,13 @@ return(fcntl(sock, F_SETFL, arg));
 /* create internet socket
    if prot = NULL then topt is timeout in ms, else topt is ttl (time to live)
 */
-int netConnect(char * remoteHostName, int portNo, int type, char * prot, int topt)
+int netConnect(char * remoteHostName, int portNo, int type, int prot, int topt)
 {
 struct addrinfo hints, *res, *res0;
 char portStr[10];
 int sock, opt;
-#ifdef WIN_32
-unsigned long arg = 1;
+#if defined(WINDOWS) || defined(EMSCRIPTEN)
+u_long arg = 1;
 #else
 int arg, value;
 socklen_t socklen = sizeof(sock);
@@ -541,9 +547,9 @@ if((sock = socket(ADDR_FAMILY, type, 0)) == INVALID_SOCKET)
     return(SOCKET_ERROR);
     }
 
-if(prot == NULL) /* topt is timeout in millisecs */
+if(prot == 0) /* topt is timeout in millisecs */
     {
-#ifdef WIN_32
+#ifdef WINDOWS
     if(ioctlsocket(sock, FIONBIO, &arg) != 0)
         {
         netErrorIdx = ERR_INET_CANNOT_CHANGE_SOCK_BLOCK;
@@ -558,18 +564,18 @@ if(prot == NULL) /* topt is timeout in millisecs */
         }
 #endif
     }
-else if(*prot == 'M' || *prot == 'B') 
+else if(prot == 'M' || prot == 'B') 
     {
     sinlen = (ADDR_FAMILY == AF_INET6) ? sizeof(struct in6_addr) : sizeof(struct in_addr);
 
-    if(*prot == 'M')
+    if(prot == 'M')
         {
         setsockopt(sock, 0, IP_MULTICAST_IF, (const void *)defaultInAddr, sinlen);
         opt = topt; /* ttl time to live */
         setsockopt(sock, 0, IP_MULTICAST_TTL, (const void *)&opt, sizeof(opt));
         }
 
-    if(*prot == 'B')
+    if(prot == 'B')
         {
         opt = 1;
         setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (const void *)&opt, sizeof(opt));
@@ -595,7 +601,7 @@ for(res = res0; res; res = res->ai_next)
     result = connect(sock, res->ai_addr, res->ai_addrlen);
     if(result < 0)
         {
-#ifdef WIN_32
+#ifdef WINDOWS
         if(WSAGetLastError() == WSAEWOULDBLOCK)
 #else
         if(errno == EINPROGRESS) 
@@ -606,7 +612,7 @@ for(res = res0; res; res = res->ai_next)
                 netErrorIdx = result < 0 ? ERR_INET_CONNECT_FAILED : ERR_INET_TIMEOUT;
                 goto CONNECT_FAILED;
                 }
-#ifndef WIN_32 
+#ifndef WINDOWS 
             getsockopt(sock, SOL_SOCKET, SO_ERROR, (void*)&value, &socklen); 
             if (value) 
                 { 
@@ -746,13 +752,13 @@ int netAccept(int listenSock)
 int sock, family;
 struct sockaddr * dest_sin;
 socklen_t dest_slen;
-#ifndef WIN_32
+#ifndef WINDOWS
 struct sockaddr_un dest_sun;
 #endif
 
 family = getSocketFamily(listenSock);
 
-#ifndef WIN_32
+#ifndef WINDOWS
 if(family == AF_UNIX)
     {
     dest_slen = sizeof(struct sockaddr_un);
@@ -917,7 +923,7 @@ char * waitFor;
 ssize_t bytesReceived;
 size_t length;
 int found;
-STREAM netStream = {0, NULL, NULL, 0, 0};
+STREAM netStream = {NULL, NULL, 0, 0, 0};
 char chr;
 CELL * cell;
 
@@ -1047,7 +1053,7 @@ params = getInteger(params, &portNo);
 params = getInteger(params, (UINT *)&readSize);
 if(params != nilCell)
     {
-    params = getInteger64(params, &wait);
+    params = getInteger64Ext(params, &wait, TRUE);
     if(params != nilCell)
         getString(params, &ifaddr);
     }
@@ -1196,13 +1202,14 @@ char * option = NULL;
 char * mcAddr = NULL;
 int sock, type; 
 int sockopt = 0;
+int opt;
 CELL * cell;
  
 type = SOCK_STREAM;
 cell = evaluateExpression(params);
 params = params->next;
 
-#ifndef WIN_32
+#ifndef WINDOWS
 if(cell->type == CELL_STRING)
     {
     if((sock = netListenLocal((char *)cell->contents)) == SOCKET_ERROR)
@@ -1221,23 +1228,26 @@ if(params != nilCell)
     if(params != nilCell)
         {
         params = getString(params, &option);
-        *option = toupper(*option);
-        if(*option == 'U')
+        opt = toupper(*option);
+        if(opt == 'U')
             type = SOCK_DGRAM;
-#ifndef WIN_32
-        else if(*option == 'D')
+#ifndef WINDOWS
+        else if(opt == 'D')
             {
             type = SOCK_RAW;
             sockopt = IPPROTO_DIVERT;
             }
 #endif
-        else if(*option == 'M')
+        else if(opt == 'M')
             {
             type = SOCK_DGRAM;
             mcAddr = ifAddr;
             ifAddr = NULL;
             }
+        else
+            errorProc(ERR_INVALID_OPTION);
         }
+            
     }
     
 
@@ -1248,7 +1258,7 @@ if((sock = netListenOrDatagram((int)portNo, type, ifAddr, mcAddr, sockopt))
 return(stuffInteger(sock));
 } 
 
-#ifndef WIN_32
+#ifndef WINDOWS
 int netListenLocal(char * name)
 {
 int sock;
@@ -1352,8 +1362,8 @@ return(sock);
 CELL * p_netPeek(CELL * params)
 {
 UINT sock;
-#ifdef WIN_32
-unsigned long result;
+#ifdef WINDOWS
+u_long result;
 #else
 int result;
 #endif
@@ -1413,7 +1423,7 @@ else if(isList(cell->type))
 else return(errorProcExt(ERR_LIST_OR_NUMBER_EXPECTED, params));
 
 params = getString(params->next, &mode);
-getInteger64(params, &wait);
+getInteger64Ext(params, &wait, TRUE);
 
 tmvPtr = (wait == -1) ? NULL : &timeOut;
 timeOut.tv_sec = wait/1000000;
@@ -1430,6 +1440,7 @@ else if(*mode == 'w')
     setSize = select(FD_SETSIZE, NULL, &socketSet, NULL, tmvPtr);
 else if(*mode == 'e')
     setSize = select(FD_SETSIZE, NULL, NULL, &socketSet, tmvPtr);
+else return(errorProcExt2(ERR_INVALID_PARAMETER, stuffString(mode)));
 
 if(setSize >= 0)
     {
@@ -1446,11 +1457,11 @@ if(setSize >= 0)
     while(sockListPtr != nilCell)
         {
         sockListPtr = getIntegerExt(sockListPtr, &socket, FALSE);
-        printf("testing %lu\n", socket);
+        /* printf("testing %lu\n", socket); */
         if(FD_ISSET(socket, &socketSet))
             {
             addList(list, stuffInteger(socket));
-            printf("adding %lu\n", socket);
+            /* printf("adding %lu\n", socket); */
             }
         }
 
@@ -1469,7 +1480,7 @@ void writeLog(char * text, int newLine)
 int handle;
 
 
-#ifdef WIN_32
+#ifdef WINDOWS
 handle = open(logFile, O_RDWR | O_APPEND | O_BINARY | O_CREAT, S_IREAD | S_IWRITE);
 #else
 handle = open(logFile, O_RDWR | O_APPEND | O_BINARY | O_CREAT,
@@ -1479,7 +1490,11 @@ handle = open(logFile, O_RDWR | O_APPEND | O_BINARY | O_CREAT,
 if(write(handle, text, strlen(text)) < 0) return;
 if(newLine) 
     if(write(handle, &LINE_FEED, LINE_FEED_LEN) < 0) return;
+#ifdef WINDOWS
+_close(handle);
+#else
 close(handle);
+#endif
 }
 
 
@@ -1494,7 +1509,7 @@ text[79] = 0;
 
 if(!reconnect)
     {
-#ifndef WIN_32
+#ifndef WINDOWS
     if(port != 0)
         sock = netListenOrDatagram(port, SOCK_STREAM, NULL, NULL, 0);
     else
@@ -1528,6 +1543,8 @@ snprintf(text, 79, "Connected to %s on %s", name, ctime(&t));
 /* printf(text); */
 writeLog(text, 0);
 
+setenv("REMOTE_ADDR", name, 1);
+
 return(fdopen(connection, "r+")); 
 }
 
@@ -1563,7 +1580,7 @@ int ready;
 int sock;
 size_t size, count = 0;
 ssize_t bytes;
-long timeOut = 60000; /* milli secs */
+INT timeOut = 60000; /* milli secs */
 int start, elapsed = 0;
 CELL * result;
 STREAM * netStream;
@@ -1629,13 +1646,13 @@ else
     }
 
 
-#ifndef WIN_32
+#ifndef WINDOWS
 if(port != 0)
-    sock = netConnect(host, (int)port, SOCK_STREAM, NULL, timeOut);
+    sock = netConnect(host, (int)port, SOCK_STREAM, 0, timeOut);
 else
     sock = netConnectLocal(host);
 #else
-sock = netConnect(host, (int)port, SOCK_STREAM, NULL, timeOut);
+sock = netConnect(host, (int)port, SOCK_STREAM, 0, timeOut);
 #endif
 
 if(sock == SOCKET_ERROR)
@@ -1917,14 +1934,13 @@ snprintf(str, 40, "%s", (numError > MAX_NET_ERROR) ? UNKNOWN_ERROR : netErrorMsg
 return(result);
 }
 
-#ifdef NET_PING
+#ifndef NO_NET_PING
 /* net-ping 
 
    Undocumented boolean flag parameter (after count) puts error info
    into the result list when a packet could not be sent.
 */
 
-
 CELL * p_netPing(CELL * params)
 {
 CELL * address;
@@ -2219,7 +2235,7 @@ answer = ~sum;
 return (answer);
 }
 
-#endif /* NET_PING */
+#endif /* NO_NET_PING */
 
 
 /* check socket for readability or error 
@@ -2266,7 +2282,7 @@ struct pseudohdr
     };
 
 
-#ifndef WIN_32
+#ifndef NO_NET_PACKET
 unsigned short pseudo_chks( struct ip * iph, char * packet, char * header, int data_offset);
 /* (net-packet str-packeA [int-config-flags]) */
 
@@ -2314,9 +2330,15 @@ switch(iph->ip_p)
   {
   case IPPROTO_TCP:
     tcph = (struct tcphdr *)(packet + sizeof(struct ip));
+#ifdef ANDROID
+    dport = tcph->dest;
+    if(tcph->check == 0) 
+        tcph->check = pseudo_chks(iph, packet, (char *) tcph, tcph->doff * 4);
+#else
     dport = tcph->th_dport;
     if(tcph->th_sum == 0) 
         tcph->th_sum = pseudo_chks(iph, packet, (char *) tcph, tcph->th_off * 4);
+#endif
     break;
   case IPPROTO_UDP:
     udph = (struct udphdr *)(packet + sizeof(struct ip));
@@ -2385,12 +2407,11 @@ checksum = in_cksum((unsigned short *)pseudoh, sizeof(struct pseudohdr) + segmen
 #endif
 return(checksum);
 }
-#endif
-
+#endif /* NO_NET_PACKET */
 
 /* ------------------ socket->filestream stuff for win32 ------------------------*/
 
-#ifdef WIN_32
+#ifdef WINDOWS
 extern int IOchannelIsSocketStream;
 
 /*
@@ -2460,7 +2481,7 @@ return(chr);
 }
 
 
-char * win32_fgets(char * buffer, int  size, FILE * fPtr)
+char * win32_fgets(char * buffer, int size, FILE * fPtr)
 {
 int bytesReceived = 0;
 char chr;
@@ -2486,6 +2507,11 @@ while(bytesReceived < size)
 return(buffer);
 }
 
-#endif
+#endif /* WINDOWS */
 
+#else /* for EMSCRIPTEN define dummy writeLog() */
+
+void writeLog(char * text, int newLine) { return; }
+
+#endif /* ifndef EMSCRIPTEN */
 /* eof */
