@@ -1,6 +1,6 @@
 /* nl-string.c
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,9 @@
 #include "protos.h"
 
 extern SYMBOL * sysSymbol[];
+extern CELL * countCell;
 extern int bigEndian;
+extern int pagesize;
 
 #define OVECCOUNT (MAX_REGEX_EXP * 3)    /*  max sub expressions in PCRE */
 
@@ -56,9 +58,6 @@ CELL * substring(char * string, ssize_t slen, ssize_t offset, ssize_t len)
 if(offset < 0)
     {
     offset = slen + offset;
-/* before 10.1.11
-    if(offset < 0) offset = 0;
-*/
     if(offset < 0)
         return(errorProc(ERR_STRING_INDEX_INVALID));
     }
@@ -101,7 +100,7 @@ ssize_t foundPosition;
 
 char * searchString;
 char * buffer;
-long options;
+INT options;
 size_t len;
 
 params = getInteger(params, (UINT *)&fileHandle);
@@ -113,13 +112,15 @@ if(params != nilCell)
     {
     next = params->next;
     params = evaluateExpression(params);
-    if(isNumber(params->type))
-        getIntegerExt(params, (UINT *)&options, FALSE); 
+    if(isNumber(params->type) || params->type == CELL_STRING)
+        /* 10.6.1 also accept string for options */
+        parseRegexOptions(params, (UINT *)&options, FALSE);
     else
         {
         flag = (params->type != CELL_NIL);
         if(next != nilCell)
-            getInteger(next, (UINT *)&options);
+            /* 10.6.1 also accept string for options */
+            parseRegexOptions(next, (UINT *)&options, TRUE);
         }
     }
 
@@ -158,7 +159,7 @@ if(position == -1) return(nilCell);
 lseek((int)fileHandle, foundPosition, SEEK_SET);
 #ifdef LFS
 result = foundPosition;
-return(stuffFloat(&result));
+return(stuffFloat(result));
 #else
 return(stuffInteger(foundPosition));
 #endif
@@ -177,10 +178,7 @@ ssize_t p;
 
 ptr = (char*)cell->contents;
 
-params = evaluateExpression(params);
-if(!isNumber(params->type))
-    errorProcExt(ERR_STRING_INDEX_INVALID, params);
-getIntegerExt(params, (UINT *)&index, FALSE);
+getInteger(params, (UINT *)&index);
 
 stringCell = cell;
 if(cell->aux == 1 && (index == 0 || index == -1)) 
@@ -495,11 +493,6 @@ size =  wstr_utf8(utf8str, unicode, size * UTF8_MAX_BYTES + 1);
 utf8str = reallocMemory(utf8str, size + 1);
 
 free(unicode);
-/*
-cell = getCell(CELL_STRING);
-cell->contents = (UINT)utf8str;
-cell->aux = size + 1;
-*/
 return(makeStringCell(utf8str, size));
 #endif
 
@@ -530,6 +523,11 @@ if(*fmt == '%')
     goto PARSE_FORMAT;
     }
 
+/* get optional decimal format spec */
+/* this GNU extension does not work on FreeBSD, OpenBSD */
+/* works on OSX 10.9, Linux and Unix with GNU libc or alike */
+if(*fmt == '\'') fmt++;
+
 /* get width spec */
 
 /* force + before numbers */
@@ -602,10 +600,8 @@ if(*fmt == 's')
     return(++fmt);
     }
 
-#ifndef WIN_32
-#ifdef TRU64 /* supporting ld, li, lu, lx, lX formats */
-if(*fmt == 'l' &&  
-        (*(fmt + 1) == 'd' || *(fmt + 1) == 'i' || *(fmt + 1) == 'u' || *(fmt + 1) =='x' || *(fmt + 1) == 'X'))
+/* supporting ld, li, lu, lx, lX formats */
+if(*fmt == 'l' &&  (*(fmt + 1) == 'd' || *(fmt + 1) == 'i' || *(fmt + 1) == 'u' || *(fmt + 1) =='x' || *(fmt + 1) == 'X'))
     {
 #ifndef NEWLISP64
     *type = CELL_INT64;
@@ -614,9 +610,9 @@ if(*fmt == 'l' &&
 #endif
     return(fmt+2);
     }
-#else /* all other UNIX suporting lld, llu, llx, llX formats */
-if(*fmt == 'l' && *(fmt + 1) == 'l' && 
-        (*(fmt + 2) == 'd' || *(fmt + 2) == 'u' || *(fmt + 2) =='x' || *(fmt + 2) == 'X'))
+
+#ifndef TRUE64 /* UNIX except TRUE64 and WINDOWS MINGW suporting lld, llu, llx, llX formats */
+if(*fmt == 'l' && *(fmt + 1) == 'l' && (*(fmt + 2) == 'd' || *(fmt + 2) == 'u' || *(fmt + 2) =='x' || *(fmt + 2) == 'X'))
     {
 #ifndef NEWLISP64
     *type = CELL_INT64;
@@ -626,7 +622,8 @@ if(*fmt == 'l' && *(fmt + 1) == 'l' &&
     return(fmt+3);
     }
 #endif
-#else /* MinGW uses MS conventions */
+
+#ifdef WINDOWS /* on MINGW also support MS conventions */
 if(memcmp(fmt, "I64", 3) == 0 &&
         (*(fmt + 3) == 'd' || *(fmt + 3) == 'u' || *(fmt + 3) =='x' || *(fmt + 3) == 'X'))
     {
@@ -659,7 +656,7 @@ char * format;
 char * fmt;
 char * nextfmt;
 CELL * cell;
-STREAM fmtStream = {0, NULL, NULL, 0, 14};
+STREAM fmtStream = {NULL, NULL, 0, 14, 0};
 int fType;
 double floatNum;
 UINT intNum;
@@ -720,7 +717,7 @@ while(params->type != CELL_NIL)
     if(fType == CELL_INT64)
         {
         if(isNumber(cell->type))
-            cell = getInteger64(cell, &bigNum);
+            cell = getInteger64Ext(cell, &bigNum, TRUE);
         else goto FORMAT_DATA_ERROR;
 
         varPrintf((UINT)&fmtStream, fmt, bigNum);
@@ -738,7 +735,11 @@ while(params->type != CELL_NIL)
             floatNum = *(double *)&cell->contents;
 #endif
         else if(cell->type == CELL_LONG)
-            floatNum = (long)cell->contents;
+            floatNum = (INT)cell->contents;
+#ifdef BIGINT
+        else if(cell->type == CELL_BIGINT)
+            floatNum = bigintCellToFloat(cell);
+#endif
         else goto FORMAT_DATA_ERROR;
 
         varPrintf((UINT)&fmtStream, fmt, floatNum);
@@ -812,7 +813,7 @@ if(stream->handle != 0)
 
 char * cellToString(CELL * cell, size_t * size, int quoteFlag)
 {
-static STREAM strStream = {0, NULL, NULL, 0, 0};
+static STREAM strStream = {NULL, NULL, 0, 0, 0};
 
 if(cell->type == CELL_STRING)
     {
@@ -875,7 +876,7 @@ stream->position = newPosition;
 
 char * readStreamText(STREAM * stream, int * size)
 {
-STREAM outStream = {0, NULL, NULL, 0, 0};
+STREAM outStream = {NULL, NULL, 0, 0, 0};
 ssize_t findPos = -1;
 size_t searchLen;
 char * result;
@@ -1099,7 +1100,7 @@ CELL * p_integer(CELL * params)
 {
 char * intString;
 INT64 num;
-long base;
+INT base;
 CELL * deflt, * cell;
 INT64 result;
 
@@ -1110,7 +1111,12 @@ if(cell->type == CELL_STRING)
     intString = (char *)cell->contents;
 else if(isNumber(cell->type))
     {
-    getInteger64(cell, &num);
+#ifdef BIGINT
+    if(cell->type == CELL_BIGINT)
+        num = bigintToInt64(cell);
+    else
+#endif
+    getInteger64Ext(cell, &num, FALSE);
     return(stuffInteger64(num));
     }
 else 
@@ -1139,7 +1145,12 @@ else if(!isDigit((unsigned char)*intString))
         goto INT_DEFAULT;
     }
 
-
+else if(*intString == '0' && (*(intString + 1) == 'b' || *(intString + 1) == 'B'))
+    {
+    intString += 2;
+    base = 2;
+    }
+     
 #ifdef TRU64 
 result = strtoul(intString, NULL, base); 
 #else 
@@ -1165,8 +1176,13 @@ if(cell->type == CELL_STRING)
     fltString = (char *)cell->contents;
 else if(isNumber(cell->type))
     {
+#ifdef BIGINT
+    if(cell->type == CELL_BIGINT)
+        value = bigintCellToFloat(cell);
+    else
+#endif
     getFloat(cell, &value);
-    return(stuffFloat(&value));
+    return(stuffFloat(value));
     }
 else
     return(copyCell(evaluateExpression(deflt)));
@@ -1181,7 +1197,7 @@ if(!isDigit((unsigned char)*fltString))
     }
 
 value = atof(fltString);
-return( stuffFloat(&value) );
+return( stuffFloat(value) );
 }
 
 
@@ -1192,7 +1208,7 @@ char number[32];
 SYMBOL * context;
 SYMBOL * sPtr;
 CELL * cell;
-#ifdef WIN_32
+#ifdef WINDOWS
 char * fmt = "%I64d";
 #endif
 
@@ -1210,11 +1226,11 @@ switch(cell->type)
         snprintf(number, 30, "%ld", *(INT64 *)&cell->aux); 
 #else
 
-#ifdef WIN_32
+#ifdef WINDOWS
         snprintf(number, 30, fmt, *(INT64 *)&cell->aux);
 #else
         snprintf(number, 30, "%lld", *(INT64 *)&cell->aux);
-#endif /* WIN_32 */
+#endif /* WINDOWS */
 
 #endif /* TRU64 */
         token = number;
@@ -1230,6 +1246,8 @@ switch(cell->type)
         break;
     case CELL_STRING:
         token = (char*)cell->contents;
+        if((cell->aux - 1) > MAX_SYMBOL)
+            errorProcExt(ERR_STRING_TOO_LONG, cell);
         break;
     case CELL_SYMBOL:
         sPtr = (SYMBOL*)cell->contents;
@@ -1262,7 +1280,7 @@ return(stuffSymbol(sPtr));
 
 CELL * p_symbolSource(CELL * params)
 {
-STREAM strStream = {0, NULL, NULL, 0, 0};
+STREAM strStream = {NULL, NULL, 0, 0, 0};
 CELL * cell;
 
 openStrStream(&strStream, MAX_STRING, 0);
@@ -1276,7 +1294,7 @@ return(cell);
 CELL * p_string(CELL * params)
 {
 CELL * cell;
-STREAM strStream = {0, NULL, NULL, 0, 14};
+STREAM strStream = {NULL, NULL, 0, 14, 0};
 
 strStream.buffer = strStream.ptr = callocMemory(15);
 prettyPrintFlags |= PRETTYPRINT_STRING;
@@ -1331,7 +1349,6 @@ if(params->type == CELL_FLOAT)
 #endif
 
 return(params->contents);
-
 }
 
 CELL * p_getChar(CELL * params)
@@ -1341,7 +1358,36 @@ return(stuffInteger((UINT)*(char*)getAddress(params)));
 
 CELL * p_getString(CELL * params)
 {
-return(stuffString((char *)getAddress(params)));
+UINT size, len;
+UINT address;
+UINT offset = 0;
+int inc = 1;
+char * limit;
+char zeros[4] = {0,0,0,0};
+
+address = getAddress(params);
+/* (get-string <address>) ; get zero terminated ASCII or UTF-8 */ 
+if(params->next == nilCell) 
+    return(stuffString((char *)address));
+
+params = getInteger(params->next, &size);
+/* (get-string <address> <int-bytes>) ; get no bytes binary data */
+if(params == nilCell) 
+    return(stuffStringN((char *)address, size));
+
+getStringSize(params, &limit, (size_t *)&len, TRUE);
+/* (get-string <address> <int-bytes> <str-limit>) ; get until limit or bytes read */
+if(len == 4 || len == 2) if(memcmp(limit, zeros, len) == 0) inc = len;
+while(offset < size)
+    {
+    if(offset < (size - len) &&
+        memcmp((char *)(address + offset), limit, len) == 0) break;
+    offset += inc;
+    }
+
+/* printf("size: %ld, len: %ld, inc: %d, offset: %ld\n", size, len, inc, offset); */
+
+return(stuffStringN((char *)address, offset));
 }
 
 CELL * p_getInteger(CELL * params)
@@ -1351,19 +1397,14 @@ return(stuffInteger(*(int *)getAddress(params)));
 
 CELL * p_getLong(CELL * params)
 {
-#ifndef NEWLISP64
 return(stuffInteger64(*(INT64 *)getAddress(params)));
-#else
-return(stuffInteger(*(UINT *)getAddress(params)));
-#endif
 }
 
 CELL * p_getFloat(CELL * params)
 {
-return(stuffFloat((double*)getAddress(params)));
+return(stuffFloat(*(double*)getAddress(params)));
 }
 
-
 CELL * p_address(CELL * params)
 {
 if(params == nilCell)
@@ -1484,11 +1525,11 @@ CELL * result;
 char * string;
 char * separator;
 char token[MAX_STRING];
-STREAM stream = {0, NULL, NULL, 0, 0};
+STREAM stream = {NULL, NULL, 0, 0, 0};
 int tklen;
 size_t srclen;
 /* PCRE stuff */
-long options = 0;
+INT options = 0;
 pcre *re = NULL;
 
 params = getStringSize(params, &string, &srclen, TRUE);
@@ -1498,7 +1539,8 @@ else
     params = getString(params, &separator);
     if(params != nilCell) 
         {
-        getInteger(params, (UINT *)&options);
+        /* 10.6.1 also accept string for options */
+        parseRegexOptions(params, (UINT *)&options, TRUE);
         /* Compile the regular expression in separator */
         re = pcreCachedCompile(separator, options); 
         }
@@ -1927,7 +1969,7 @@ while( (source = parsePackFormat(source, &len, &type))!= NULL)
             *(INT64 *)&cell->aux = int32V;
 #else
             cell = getCell(CELL_LONG);
-            *(long *)&cell->contents = int32V;
+            *(INT *)&cell->contents = int32V;
 #endif
             break;
 
@@ -1958,13 +2000,13 @@ while( (source = parsePackFormat(source, &len, &type))!= NULL)
             memcpy(&floatV, pPtr, 4);
                 if(endianSwitch) swapEndian((char*)&floatV, 4);
             doubleV = floatV;
-            cell = stuffFloat(&doubleV);
+            cell = stuffFloat(doubleV);
             break;
 
         case PACK_DOUBLE:
             memcpy(&doubleV, pPtr, 8);
             if(endianSwitch) swapEndian((char*)&doubleV, 8);
-            cell = stuffFloat(&doubleV);
+            cell = stuffFloat(doubleV);
             break;
             
         case PACK_STRING:
@@ -2078,6 +2120,7 @@ return(result);
 /* ------- PRCE Perl Compatible Regular Expressions for all platforms ------- */
 /*                    see also http://www.pcre.org/                           */
 
+/* currently PCRE v.5.0 from 13-Sep-2004  -> pcre.h */
 
 /* additional regex option */
 #define REPLACE_ONCE 0x8000
@@ -2093,19 +2136,23 @@ pcre *re;
 int ovector[OVECCOUNT];
 int rc, idx;
 char * pattern;
+char * subject;
+#ifdef SUPPORT_UTF8
 char * string;
-long options = 0;
+#endif
+INT options = 0;
 UINT offset = 0;
 int len;
 size_t size;
 CELL * cell, * result, * strCell;
 
 params = getString(params, &pattern);
-params = getStringSize(params, &string, &size, TRUE);
+params = getStringSize(params, &subject, &size, TRUE);
 
 if(params != nilCell)
     {
-    params = getInteger(params, (UINT *)&options);
+    /* 10.6.1 also accept string for options */
+    params = parseRegexOptions(params, (UINT *)&options, TRUE);
     if(params != nilCell)
         getInteger(params, &offset);
     }
@@ -2117,7 +2164,7 @@ re = pcreCachedCompile(pattern, (int)options);
 rc = pcre_exec(
     re,            /* the compiled pattern */
     NULL,          /* no extra data - we didn't study the pattern */
-    string,        /* the subject string */
+    subject,       /* the subject string */
     size,          /* the length of the subject */
     (int)offset,   /* start at offset 0 in the subject */
     0,             /* default options */
@@ -2139,7 +2186,7 @@ result = cell = getCell(CELL_EXPRESSION);
 for(idx = 0; idx < rc; idx++)
     {
     len = ovector[2*idx+1] - ovector[2*idx];
-    strCell = stuffStringN(string + ovector[2*idx], len);
+    strCell = stuffStringN(subject + ovector[2*idx], len);
 
     deleteList((CELL*)sysSymbol[idx]->contents);
     sysSymbol[idx]->contents = (UINT)copyCell(strCell);
@@ -2155,8 +2202,21 @@ for(idx = 0; idx < rc; idx++)
         cell = cell->next;
         }
 
+#ifdef SUPPORT_UTF8
+    if(2048 & options)
+        cell->next = stuffInteger(utf8_wlen(subject, subject + ovector[2*idx] + 1));
+    else
+#endif
     cell->next = stuffInteger(ovector[2*idx]);
     cell = cell->next;
+#ifdef SUPPORT_UTF8
+    if(2048 & options)
+        {
+        string = (char *)strCell->contents;
+        cell->next = stuffInteger(utf8_wlen(string, string + len + 1));
+        }
+    else
+#endif
     cell->next = stuffInteger(len);
     cell = cell->next;
     }
@@ -2168,7 +2228,7 @@ return(result);
 CELL * p_regexComp(CELL * params)
 {
 char * pattern;
-long options = 0;
+INT options = 0;
 pcre * re;
 int rc;
 size_t size;
@@ -2176,7 +2236,8 @@ CELL * result;
 
 params = getString(params, &pattern);
 if(params != nilCell)
-    params = getInteger(params, (UINT *)&options);
+    /* 10.6.1 also accept string for options */
+    params = parseRegexOptions(params, (UINT *)&options, TRUE);
 
 /* Compile the regular expression in the first argument */
 re = pcreCachedCompile(pattern, (int)options);
@@ -2247,7 +2308,7 @@ if((cPattern == NULL) || (strcmp(cPattern, pattern) != 0) || (options != cacheOp
     len = strlen(pattern);
     cPattern = (char *)allocMemory(len + 1);
     memcpy(cPattern, pattern, len + 1);
-#ifdef WIN_32
+#ifdef WINDOWS
     if(re != NULL) free(re);
 #else
     if(re != NULL) (pcre_free)(re);
@@ -2337,6 +2398,7 @@ while(offset <= buffLen)
 
     resultStackIdxSave = resultStackIdx; 
 
+    countCell->contents++;
     if((cell = evaluateExpressionSafe(exprCell, &errNo)) == NULL)
         {
         freeRegex(start_rx);
@@ -2426,6 +2488,48 @@ while(regex != NULL)
     }
 }
 
+
+/* support strings in the integer-options field of:
+    directory, ends-with, find, find-all, parse, regex, replace, search, starts-with
+   instead of: (find "xyz" buff (| 1 4 2048))
+   example: (find "x.z" "abcX\nZ" "is") => 3
+   for PCRE_CASELESS, PCRE_DOTALL
+*/
+
+CELL* parseRegexOptions(CELL* params, UINT * options, int evalFlag)
+{
+CELL* cell;
+int i;
+char* buffer;
+
+if (evalFlag)
+    cell = evaluateExpression(params);
+else
+    cell = params;
+
+*options = 0;
+
+if (cell->type == CELL_STRING) 
+    {
+    buffer = (char*)cell->contents;
+    for (i = 0; i < cell->aux - 1; i ++) {
+        switch (buffer[i]) {
+            case 'i': *options |= 1; break;         /* PCRE_CASELESS */
+            case 'm': *options |= 2; break;         /* PCRE_MULTILINE */
+            case 's': *options |= 4; break;         /* PCRE_DOTALL */
+            case 'x': *options |= 8; break;         /* PCRE_EXTENDED */
+            case 'A': *options |= 16; break;        /* PCRE_ANCHORED */
+            case 'D': *options |= 32; break;        /* PCRE_DOLLAR_ENDONLY */
+            case 'U': *options |= 512; break;       /* PCRE_UNGREEDY */
+            case 'u': *options |= 2048; break;      /* PCRE_UTF8 */
+            default: return(errorProcExt(ERR_INVALID_OPTION, cell));
+            }
+        }
+    } else { getIntegerExt(cell, options, FALSE); }
+
+return (params->next);
+}
+
 /* eof */
 
 
