@@ -1,6 +1,6 @@
 /* nl-symbol.c --- symbol handling routines for newLISP
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ extern SYMBOL * orSymbol;
 SYMBOL * findInsertSymbol(char * key, int forceCreation);
 int deleteSymbol(char * key);
 void deleteContextSymbols(CELL * cell, int checkReferences);
+void changeContextCells(SYMBOL * contextPtr);
 CELL dumpSymbol(char * name);
 void collectSymbols(SYMBOL * sPtr, CELL * symbolList);
 void collectSymbolAssocs(SYMBOL * sPtr, CELL * assocList);
@@ -132,10 +133,20 @@ return(findInsertSymbol(token, LOOKUP_ONLY));
 SYMBOL * makeSafeSymbol(CELL * cell, SYMBOL * context, int flag)
 {
 char * token;
+UINT number;
 
-token = alloca(cell->aux + 1);
-*token = '_';
-memcpy(token + 1, (char *)cell->contents, cell->aux);
+if(isNumber(cell->type))
+    {
+    token = alloca(32);
+    getIntegerExt(cell, &number, FALSE);
+    snprintf(token, 31, "_%ld", number);
+    }
+else
+    {
+    token = alloca(cell->aux + 1);
+    *token = '_';
+    memcpy(token + 1, (char *)cell->contents, cell->aux);
+    }
     
 if(flag)
         return(translateCreateSymbol(token, CELL_NIL, context, TRUE));
@@ -225,6 +236,14 @@ sPtr->context = context;
 return(sPtr);
 }
 
+CELL * assignSymbol(SYMBOL * sPtr, CELL * content)
+{
+deleteList((CELL*)sPtr->contents);
+sPtr->contents = (UINT)content;
+
+return(content);
+}
+
 /* ------------------------- dump RB tree info of a symbol -------------------- */
 
 #ifdef SYMBOL_DEBUG
@@ -255,7 +274,7 @@ return(trueCell);
 
 /* ----------------------------- delete a symbol --------------------------- */
 int references(SYMBOL * sPtr, int replaceFlag);
-int externalReferences(SYMBOL * contextPtr, int replaceFlag);
+int externalContextReferences(SYMBOL * contextPtr, int replaceFlag);
 
 CELL * p_deleteSymbol(CELL * params)
 {
@@ -298,7 +317,7 @@ else if(getFlag(params))
     {
     if(cell->type == CELL_CONTEXT)
         {
-        if(externalReferences(sPtr, FALSE) > 0)
+        if(externalContextReferences(sPtr, FALSE) > 1)
             {
             sPtr->flags |= SYMBOL_PROTECTED;
             return(nilCell);
@@ -316,13 +335,11 @@ else if(getFlag(params))
 if(cell->type == CELL_CONTEXT)
     {
     deleteContextSymbols(cell, checkReferences);
-    cell->type = CELL_SYMBOL; /* demote */
-    deleteList((CELL *)sPtr->contents);
+    deleteList((CELL *)sPtr->contents); 
     sPtr->contents = (UINT)copyCell(nilCell);
     }
 else 
-    deleteFreeSymbol(sPtr, checkReferences);
-
+    deleteAndFreeSymbol(sPtr, checkReferences);
 return(trueCell);
 }
 
@@ -336,7 +353,7 @@ CELL * nextSymbol;
 context = (SYMBOL *)cell->contents;
 
 if(checkReferences)
-    externalReferences(context, TRUE);
+    externalContextReferences(context, TRUE);
 
 symbolList = getCell(CELL_EXPRESSION);
 collectSymbols((SYMBOL *)((CELL *)context->contents)->aux, symbolList);
@@ -344,15 +361,47 @@ collectSymbols((SYMBOL *)((CELL *)context->contents)->aux, symbolList);
 nextSymbol = (CELL *)symbolList->contents;
 while(nextSymbol != nilCell)
     {
-    deleteFreeSymbol((SYMBOL*)nextSymbol->contents, FALSE);
+    deleteAndFreeSymbol((SYMBOL*)nextSymbol->contents, FALSE);
     nextSymbol = nextSymbol->next;
     }
-    
+
+if(checkReferences)
+    changeContextCells(context);
+
 deleteList(symbolList);
 }
 
 
-void deleteFreeSymbol(SYMBOL * sPtr, int checkReferences)
+
+/*
+void deleteContextSymbols(CELL * cell, int checkReferences)
+{
+SYMBOL * context;
+SYMBOL * treePtr;
+CELL * symbolList;
+CELL * nextSymbol;
+
+context = (SYMBOL *)cell->contents;
+treePtr = (SYMBOL *)cell->aux;
+
+symbolList = getCell(CELL_EXPRESSION);
+collectSymbols((SYMBOL *)treePtr, symbolList);
+
+nextSymbol = (CELL *)symbolList->contents;
+while(nextSymbol != nilCell)
+    {
+    deleteAndFreeSymbol((SYMBOL*)nextSymbol->contents, FALSE);
+    nextSymbol = nextSymbol->next;
+    }
+    
+if(checkReferences)
+    externalContextReferences(context, TRUE);
+
+deleteList(symbolList);
+}
+*/
+
+void deleteAndFreeSymbol(SYMBOL * sPtr, int checkReferences)
 {
 SYMBOL * context;
 
@@ -371,23 +420,6 @@ freeMemory(sPtr->name);
 freeMemory(sPtr);
 }
 
-/*
-There is more stuff then below to free for callback closures.
-At the moment not used instead overwriting is disabled.
-*/
-/*
-void deleteFFImemory(CELL * pCell)
-{
-FFIMPORT * ffi;
-
-if(cell->type == CELL_IMPORT_FFI)
-    {
-    ffi = (FFIMPORT *)pCell->aux;
-    free(ffi->cif.arg_types);
-    free(ffi);
-    }
-}
-*/
 
 void makeContextFromSymbol(SYMBOL * symbol, SYMBOL * treePtr)
 {
@@ -446,7 +478,7 @@ while(blockPtr != NULL)
 return(count);
 }
 
-int externalReferences(SYMBOL * contextPtr, int replaceFlag)
+int externalContextReferences(SYMBOL * contextPtr, int replaceFlag)
 {
 CELL * blockPtr;
 int i, count = 0;
@@ -467,6 +499,11 @@ while(blockPtr != NULL)
                     blockPtr->contents = (UINT)nilSymbol;
                 }
             }
+        if(blockPtr->type == CELL_CONTEXT)
+            {
+            if((SYMBOL *)blockPtr->contents == contextPtr)
+                count++;
+            }
         blockPtr++;
         }
     blockPtr = blockPtr->next;
@@ -475,6 +512,31 @@ while(blockPtr != NULL)
 return(count);
 }
 
+void changeContextCells(SYMBOL * contextPtr)
+{
+CELL * blockPtr;
+int i; 
+
+blockPtr = cellMemory;
+while(blockPtr != NULL)
+    {
+    for(i = 0; i < MAX_BLOCK; i++)
+        {
+        if (blockPtr->type == CELL_CONTEXT)
+            {
+            if((SYMBOL *)blockPtr->contents == contextPtr)
+                {
+                blockPtr->type = CELL_NIL;
+                blockPtr->contents = (UINT)nilCell;
+                blockPtr->aux = (UINT)nilCell;
+                }
+            }
+        blockPtr++;
+        }
+    blockPtr = blockPtr->next;
+    }
+}
+
 /* renamed to 'term' in v.10.1.11 */
 CELL * p_term(CELL * params)
 {
