@@ -1,6 +1,6 @@
 /* nl-utf8.c --- functions for UTF-8 unicode support
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -134,7 +134,7 @@ int gcaa;
 int c;
 size_t count = 0;
 
-while((c = *utf8str++) != 0)
+while((c = *utf8str++) != 0 && utf8str < limit)
     {
     count++;
     if ((c & 0xc0) == 0xc0)
@@ -144,8 +144,10 @@ while((c = *utf8str++) != 0)
         }
     }
 
+/* now handled by while loop 10.6.1
 if(utf8str > limit)
     errorProc(ERR_INVALID_UTF8);
+*/
 
 return(count);
 }
@@ -262,7 +264,7 @@ size_t size;
 char * utf8str;
 
 params = getStringSize(params, (void *)&unicode, &size, TRUE);
-#ifdef WIN_32
+#ifdef WINDOWS
 if(getFlag(params)) /* its a MBCS string */
     return(utf8_from_mbcs((void *)unicode));
 #endif
