@@ -1,6 +1,6 @@
 /* protos.h function prototypes fo6 newLISP
 
-    Copyright (C) 2012 Lutz Mueller
+    Copyright (C) 2015 Lutz Mueller
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,19 +24,20 @@ CELL * appendNode(CELL * node, CELL * newNode);
 CELL * appendString(CELL * cell, CELL * params, char * joint, size_t jointLen, int trailJoint, int evalFlag);
 CELL * appendWriteFile(CELL * params, char * type);
 CELL * arithmetikOp(CELL * params, int operand);
-CELL * arrayList(CELL * params);
+CELL * arrayList(CELL * params, int flag);
 CELL * arrayTranspose(CELL * params);
+CELL * assignSymbol(SYMBOL * sPtr, CELL * content);
 CELL * associationsFromTree(SYMBOL * context);
 CELL * bindList(CELL * list, int flag);
-CELL * collectSelect(CELL * head, CELL * params, int evalFlag);
 CELL * compareOp(CELL * params, int operand);
 CELL * copyCell(CELL * cell);
 CELL * copyList(CELL * cell);
-CELL * defineOrMacro(CELL * params, UINT type);
+CELL * defineOrMacro(CELL * params, UINT type, int flags);
 CELL * p_deleteFile(CELL * params);
 CELL * dolist(CELL * params, int argsFlag);
 CELL * environment(void);
 CELL * errorProc(int errorNumber);
+CELL * errorProcArgs(int errorNumber, CELL * expr);
 CELL * errorProcExt(int errorNumber, CELL * expr);
 CELL * errorProcExt2(int errorNumber, CELL * expr);
 CELL * evalCheckSymbol(CELL * cell);
@@ -58,7 +59,7 @@ CELL * functionFloat(CELL * params, int operand);
 CELL * getCell(int type);
 CELL * getFloat(CELL * params, double * floatNumber);
 CELL * getInteger(CELL * params, UINT * number);
-CELL * getInteger64(CELL * params, INT64 * number);
+CELL * getInteger64Ext(CELL * params, INT64 * number, int evalFlag);
 CELL * getIntegerExt(CELL * params, UINT * number, int evalFlag);
 CELL * getListHead(CELL * params, CELL * * list);
 CELL * getEvalDefault(CELL * params, CELL * * result);
@@ -97,10 +98,13 @@ CELL * netPeerLocal(CELL * params, int peerLocalFlag);
 CELL * packFFIstruct(CELL * cell, CELL * params);
 CELL * unpackFFIstruct(CELL * cell, char * data);
 #endif
+CELL* parseRegexOptions(CELL* params, UINT * options, int evalFlag);
 CELL * setEvent(CELL * params, SYMBOL * * eventSymPtr, char * sysSymName);
 CELL * setNthStr(CELL * cellStr, CELL * new, char * insertPtr);
 CELL * subarray(CELL * array, ssize_t offset, ssize_t length);
 CELL * transposeArray(CELL * array);
+CELL * p_JSONparse(CELL * params);
+CELL * p_JSONerror(CELL * params);
 CELL * p_XMLerror(CELL * params);
 CELL * p_XMLparse(CELL * params);
 CELL * p_XMLtypeTags(CELL * params);
@@ -128,6 +132,8 @@ CELL * p_base64Enc(CELL * params);
 CELL * p_base64Dec(CELL * params);
 CELL * p_bayesTrain(CELL * params);
 CELL * p_bayesQuery(CELL * params);
+CELL * p_kmeansTrain(CELL * params);
+CELL * p_kmeansQuery(CELL * params);
 CELL * p_beta(CELL * params);
 CELL * p_betai(CELL * params);
 CELL * p_bind(CELL * params);
@@ -146,6 +152,7 @@ CELL * p_char(CELL * params);
 CELL * p_chop(CELL * params);
 CELL * p_clean(CELL * params);
 CELL * p_close(CELL * params);
+CELL * p_collect(CELL * params);
 CELL * p_colon(CELL * params);
 CELL * p_commandEvent(CELL * params);
 CELL * p_continue(CELL * params);
@@ -212,6 +219,9 @@ CELL * p_errorText(CELL * params);
 CELL * p_eval(CELL * params);
 CELL * p_evalBlock(CELL * params);
 CELL * p_evalString(CELL * params);
+#ifdef EMSCRIPTEN
+CELL * p_evalStringJS(CELL * params);
+#endif
 CELL * p_exec(CELL * params);
 CELL * p_exists(CELL * params);
 CELL * p_exit(CELL * params);
@@ -274,6 +284,7 @@ CELL * p_isEmpty(CELL * params);
 CELL * p_isFile(CELL * params);
 CELL * p_isFloat(CELL * params);
 CELL * p_isInteger(CELL * params);
+CELL * p_isBigInteger(CELL * params);
 CELL * p_isLambda(CELL * params);
 CELL * p_isLegal(CELL * params);
 CELL * p_isList(CELL * params);
@@ -299,6 +310,7 @@ CELL * p_local(CELL * params);
 CELL * p_log(CELL * params);
 CELL * p_lookup(CELL * params);
 CELL * p_lower(CELL * params);
+CELL * p_macro(CELL * params);
 CELL * p_mainArgs(CELL * params);
 CELL * p_makeDir(CELL * params);
 CELL * p_map(CELL * params);
@@ -491,12 +503,13 @@ CELL * setDefine(SYMBOL * symbol, CELL * params, int force);
 CELL * setInterDiff(CELL * params, int mode);
 CELL * setRef(CELL * params, int mode);
 CELL * strUpperLower(CELL * params, int type);
-CELL * stuffFloat(double * floatPtr);
+CELL * stuffFloat(double floatPtr);
 CELL * stuffInteger(UINT contents);
 #ifndef NEWLISP64
 CELL * stuffInteger64(INT64 contents);
 #endif
 CELL * stuffIntegerList(int argc, ...);
+CELL * stuffBigint(char * token);
 CELL * stuffString(char * string);
 CELL * stuffStringN(char * string, int len);
 CELL * stuffSymbol(SYMBOL * sPtr);
@@ -527,8 +540,13 @@ char * prompt(void);
 char * readStreamText(STREAM * stream, int * size);
 char * replaceString (char * keyStr, int keyLen, char * buff, size_t buffLen, CELL * exprCell, 
     					UINT * cnt, int options, size_t * newLen);
+#ifdef WINDOWS
 char * win32_fgets(char * buffer, int  size, FILE * fPtr);
 char * win32_realpath(const char * filepath, char * realpath);
+char * win32_getExePath(char *);
+#endif
+
+char * which(char * name, char * path);
 size_t Curl_base64_encode(const char *inp, size_t insize, char **outptr);
 size_t Curl_base64_decode(const char *src, char *dest);
 double getDirectFloat(CELL * param);
@@ -551,7 +569,7 @@ int getRegexOptions(CELL * params);
 int isLegalSymbol(char * symName);
 unsigned short in_cksum(unsigned short * addr, int len);
 int isDir(char * fileName);
-int isFile(char * fileName);
+int isFile(char * fileName, int flag);
 int main(int argc, char * argv[]);
 int makeStreamFromFile(STREAM * stream, char * fileName, size_t size, size_t offset);
 int makeStreamFromString(STREAM * stream, char * str);
@@ -560,7 +578,7 @@ INT64 microSecTime(void);
 int my_setenv(const char * varName, const char * varValue, int flag);
 int my_strnicmp(char * s1, char * s2, ssize_t size);
 int my_vasprintf(char * * buffer, const char * format, va_list argptr);
-int netConnect(char * name, int port, int type, char * protocol, int ttl);
+int netConnect(char * name, int port, int type, int protocol, int ttl);
 int netConnectLocal(char * path);
 int netAccept(int listenSock);
 int netListenLocal(char * name);
@@ -596,12 +614,12 @@ void freeCellBlocks();
 void returnBlockMemory(void);
 void collectSymbols(SYMBOL * context, CELL * list);
 void deleteArray(CELL * cell);
-void deleteFreeSymbol(SYMBOL * sPtr, int checkReferences);
+void deleteAndFreeSymbol(SYMBOL * sPtr, int checkReferences);
 void deleteList(CELL * cell);
 void deleteTagStack(void);
 void encryptPad(char * encrypted, char * data, char * key, size_t dataLen, size_t keyLen);
 void executeCommandLine(char * command, UINT outDevice, STREAM * cmdStream);
-void expandSymbol(CELL * cell, SYMBOL * sPtr);
+void expandExprSymbol(CELL * cell, SYMBOL * sPtr);
 void errorMissingPar(STREAM * stream);
 void fatalError(int errorNumber, CELL * expr, int deleteFlag);
 #ifdef FFI
@@ -658,6 +676,25 @@ size_t appendCellString(CELL * cell, char * buffer, size_t size);
 UINT * copyArray(CELL * params);
 UINT64 timediff64_us(struct timeval end, struct timeval start);
 
+#ifdef BIGINT
+int * getBigintSizeDirect(CELL * params, int * * numPtr, int * len);
+int * mulBigint(int * x, int nx, int * y, int ny, int * p, int * n);
+int * divModBigint(int * x, int nx, int * y, int ny, int rmndr, int * n);
+int * addBigint(int * x, int nx, int * y, int ny, int * sm, int * nsm);
+int * subBigint(int * x, int nx, int * y, int ny, int * sm, int * nsm);
+int cmpBigint(int * x, int nx, int * y, int ny);
+int cmpAbsBigint(int * x, int nx, int * y, int ny);
+int * strToBigint(char * str, int len, int * intlen);
+char * bigintToDigits(int * num, int n, int offset, int * slen);
+INT64 bigintToInt64(CELL * cell);
+double bigintCellToFloat(CELL * cell);
+double bigintToAbsFloat(int * num, int n);
+CELL * p_bigInt(CELL * params);
+int * intToBigint(INT64 num, int * len);
+int * floatToBigint(double fnum, int * len);
+int lengthBigint(int * num, int len);
+#endif
+
 #ifdef SUPPORT_UTF8
 int utf8_1st_len(char * utf8str);
 int utf8_wstr(int * unicode, char * utf8str, int maxwc);
